DAT programName         byte "MandyLeds130515d", 0

CON
{{
  ******* Public Notes *******

  Written By: Duane Degn
  Posted to the forum on May 17, 2013.
  
  MIT License (Use anyway you want but give me credit.)

  This program is used to control a strip of 160 LEDs.
  Each LED is controlled with a WS2801 chip.
  Not all stands of RGB LEDs use the same order to colors
  in the communication protocol. JonnyMac's code allows
  the red and blue channels to be switch. I found my
  strand of 160 LEDs purchased off eBay had the red channel
  switched with the green channel. The method "AdjustStripBrightness"
  swaps these channels as it adjusts the brightness level of each
  LED.

  I have objects to control a FM clock module and real time clock. These
  objects are not used.

  The object "WearableKeypad. . ." as code to control the keypad's LED.
  I originally had this code working but I've done something to break it.
  The LED on the wearable keypad does not light with this present version
  of the program.

  More information about this project my be found on in a thread in the
  Projects Forum at Parallax.
  
  http://forums.parallax.com/showthread.php/147963-Mood-Enhancing-LEDs
  
   
}}
{
  ******* Private Notes *******
  130429f Thinking about sub modes but go back to single level
  modes for now.
  130430b Compiles. Not tested yet.
  30g Kind of works a little. Figured out how to clear LCD.
  Delay doesn't change yet.
  30i Change keypad LED timing to milliseconds instead of PWM
  cycles.
  0501b Slide pins assignments to make room for clock on P26&27
  0505a Everything pluged in?
  08a Work on button debounce.
  08e Clock finally works. Fixed LcdPad method. Button bounce seems to
  be working correctly.
  09b Work on strip LED programming. Comment out Xmas LED stuff for now.
  09d LED strip doesn't work yet.
  09e Coming along.
  09f Something is wrong but it looks cool.
  Fades from red to green over length of strip.
  09h Add rollover option to parameter changes. Clock still has
  trouble. Time not saved between power on.
  09i Try to fix time. Only rainbow mode seems to work right now.
  09j Fixed button fast mode but broke clock. Colors still don't work.
  10a Clock seems to be working again.
  10b Colors are working.
  10d Start trying to figure out how the strip delay will work.
  10e Change stripDelay to stripSpeed. The delay will be calculated
  from the speed. Zero speed will be off.
  10e Add table to convert from speed to delay to give a bit of logrithmic
  curve.
  10e Compiles but no scrolling LEDs.
  10f Add debug lock.
  11a Working pretty well. Scrolling is working. There is still a problem
  with the clock and I doubt the radio works.
  LEDs can scroll, change color, change brightness. The size of patterns
  can be stretched and the scroll speed changed.
  There appears to be a glitch in the program causing one LED to be
  the wrong color.
  11c I think the glitch is do to a bad solder connection on one of
  the LEDs. Start of scroll speed fixed.
  11d Try to add Nordic code. Nordic didn't work. Transmission count was
  always the same but the IRQ pin kept going low.
  11e Nordic fob seems to be working.
  11f Fob button presses registering correctly. Still problem of
  fob sending two presses closs together.
  11f Need a debounce timer for the Nordic Fob.
  11g Add parts to CheckForModeChange method in attempt to keep track
  of Nordic fob quick presses. Abandon much of new stuff. I'll likey
  need a separate method.
  12b Kind of works with the fob. Key presses faster than 1 second
  will cause large changes in the values.
  12c Works pretty well. Probably good enough to bring.
  12d Start changing mode selection.
  12e Continue changing menu functions.
  13b Abandon menu changes for now.
  13c Revert to 12c. Try to add Larson scanner.
  14b Compiles but has lots of problems. Add ability to turn off piezo.
  15c Larsen scanner seems to be working correctly. Save this version
  before making any changes.
  15d It would be nice if the Larsen scanner LED faded faster. The center
  LEDs look too bright. Try a logrithmic fade instead of linear.
  15d Added FadeColors method. The Larsen scanner fading looks better
  with dark background this way. A white background washes out the
  fade of the foreground color.
  
  
}
CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal
' _xinfreq = 6_250_000                                          ' use 6.25MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000


CON

  SPOKES = 27 '20 '6
  LEDS_PER_SPOKE = 6 '4
  SIZE_OF_SCANNER_GROUP = 4
  ' color enumeration
  '#0, RED, ORANGE, YELLOW, GREEN, BLUE, INDIGO, VIOLET, WHITE

   ' color enumeration tri color LEDs
  #0, BLACK, RED, ORANGE, YELLOW, GREEN, BLUE, VIOLET, WHITE, INDIGO

   ' stripColor enumeration 
  #0, OFF_STRIP, RED_STRIP, ORANGE_STRIP_STRIP, YELLOW_STRIP, GREEN_STRIP, BLUE_STRIP, {
    } VIOLET_STRIP, WHITE_STRIP, INDIGO_STRIP, RAINBOW_STRIP


  #0, RED_PORTION, GREEN_PORTION, BLUE_PORTION

CON ' LED Colors

  RED_R = $FF
  RED_G = 0
  RED_B = 0
  ORANGE_R = $FF
  ORANGE_G = $2F
  ORANGE_B = 0
  YELLOW_R = $FF
  YELLOW_G = $6F
  YELLOW_B = 0
  GREEN_R = 0
  GREEN_G = $FF
  GREEN_B = 0
  BLUE_R = 0
  BLUE_G = 0
  BLUE_B = $FF
  INDIGO_R = $3F
  INDIGO_G = 0
  INDIGO_B = $FF
  VIOLET_R = $8F
  VIOLET_G = 0
  VIOLET_B = $FF
  WHITE_R = $FF
  WHITE_G = $CF
  WHITE_B = $3F
  
CON

  ORANGE_COLOR = $FF_1F_00 '$FF_3F_00 '$FF_6F_00 ' 
  INDIGO_COLOR = $3F_00_FF
  YELLOW_COLOR = $FF_68_00 '$FF_6F_00 '$FF_7F_00 '$FF_5F_00 
  TERMINATOR = -1
  ADJUSTABLE_PARAMETERS = 9
  

  DEFAULT_RADIO_VOLUME = 4
  MIN_RADIO_VOLUME = 0
  MAX_RADIO_VOLUME = 15
  SMALL_CHANGE_RADIO_VOLUME = 1
  BIG_CHANGE_RADIO_VOLUME = 2

 
  MIN_RADIO_FREQ = 87_000_000
  MAX_RADIO_FREQ = 108_000_000 
  MIN_RADIO_STEPS = 0
  MAX_RADIO_STEPS = 100 
  SMALL_CHANGE_RADIO_STEPS = 1
  BIG_CHANGE_RADIO_STEPS = 1

CON

  'DEFAULT_XMAS_MOVEMENT_DELAY = 50
  'DEFAULT_STRIP_MOVEMENT_DELAY = 50
  'MAX_STRIP_MOVEMENT_DELAY = (DEFAULT_STRIP_MOVEMENT_DELAY * 100) + 1
  'MIX_STRIP_MOVEMENT_DELAY = DEFAULT_STRIP_MOVEMENT_DELAY * -100
  DEFAULT_STRIP_SPEED = 88 '95
  MAX_STRIP_SPEED = 100
  MIN_STRIP_SPEED = -100
  'STRIP_SPEED_DELAY_SLOPE = 20
  'STRIP_SPEED_DELAY_INTERCEPT = 2000

  
  MAX_STRIP_BRIGHTNESS = $FF
  DEFAULT_STRIP_BRIGHTNESS = MAX_STRIP_BRIGHTNESS / 2
  DEFAULT_VERY_DIM = $F
  MAX_XMAS_BRIGHTNESS = $CC
  DEFAULT_XMAS_BRIGHTNESS = MAX_XMAS_BRIGHTNESS
  DEFAULT_XMAS_SPEED = 90 '95
  MIN_XMAS_SPEED = MIN_STRIP_SPEED
  MAX_XMAS_SPEED = MAX_STRIP_SPEED
  DEFAULT_STRIP_SIZE = 10
  MIN_LARSEN_BUFFER_SIZE = 4
  DEFAULT_LARSEN_SCANNERS = 4
  MIN_LARSEN_SCANNERS = 1
  MAX_LARSEN_SCANNERS = CHANNELS / MIN_LARSEN_BUFFER_SIZE
  DEFAULT_LARSEN_SIZE = 3
  MIN_LARSEN_SIZE = 1
  MAX_LARSEN_SIZE = CHANNELS
                                             
CON

  CHANNELS = 160 '32 'SPOKES * LEDS_PER_SPOKE'32 '24 '7
  LAST_CHANNEL  = CHANNELS - 1
  'MAX_BUFFER_SIZE = CHANNELS + LARGEST_POSSIBLE_PATTERN '(CHANNELS * 3) / 2
  MAX_BUFFER_SIZE = LARGEST_POSSIBLE_PATTERN 
  SHORT_PAUSE = 20
  ROTATE_PAUSE = 300 '500
  LARGEST_POSSIBLE_PATTERN = CHANNELS * 2
  SINGLE_COLOR_PAT_SIZE = 5
  
CON ' I/O Pin Assignments

  RX1     = 31                                                  ' programming / debug port
  TX1     = 30
  
  SDA     = 29                                                  ' eeprom
  SCL     = 28

  ' WS2801 connections
  ' -- requires 3.3v to 5v translation (e.g., TC4427, 74x245, etc.)
  
  CK      =  10 '9
  SD      =  11 '8

     'Pins to Hong Kong Nordic nRF24L01+
  '_HkVss = 7
  NORDIC_BASE_PIN = 16
  NORDIC_IRQ = NORDIC_BASE_PIN + 0
  NORDIC_MISO = NORDIC_BASE_PIN + 1
  NORDIC_MOSI = NORDIC_BASE_PIN + 2
  NORDIC_SCK = NORDIC_BASE_PIN + 3
  NORDIC_CSN = NORDIC_BASE_PIN + 4
  NORDIC_CE = NORDIC_BASE_PIN + 5  
  ADC_CLOCK = NORDIC_BASE_PIN + 6
  ADC_DATA = NORDIC_BASE_PIN + 7
  ADC_CS = NORDIC_BASE_PIN + 8
  PIEZO_PIN = NORDIC_BASE_PIN + 9
  
  {SparkFun Module
  NORDIC_BASE_PIN = 0
  NORDIC_IRQ = NORDIC_BASE_PIN + 10
  NORDIC_MISO = NORDIC_BASE_PIN + 8
  NORDIC_MOSI = NORDIC_BASE_PIN + 6
  NORDIC_SCK = NORDIC_BASE_PIN + 4
  NORDIC_CSN = NORDIC_BASE_PIN + 2
  NORDIC_CE = NORDIC_BASE_PIN + 0
  }
  
{
  _LedPin = 18
  _ButtonPadP52 = 19            ' Pulled high with 10K ohm resistor
  _ButtonPadP53 = 20            ' Pulled high with 10K ohm resistor
  _ButtonPadP51 = 21            ' Pulled high with 10K ohm resistor
}
  LED_PIN = 12 '14 '16
  PAD_PIN_52 = LED_PIN + 3 '15 '17 '17
  PAD_PIN_53 = LED_PIN + 2 '14 '16 '18
  PAD_PIN_51 = LED_PIN + 1 '13 '15 '19

  'FIRST_NORDIC_PIN = 16
  
  LCD_RX_PIN = 9                ' P9 is dead
  LCD_TX_PIN = 8

  XMAS_0 = 6 '10 '12
  XMAS_1 = 7 '11 '13
  
  RADIO_CLOCK = 26
  RADIO_DATA = 27
  
  I2C_CLOCK = RADIO_CLOCK   
  I2C_DATA = RADIO_DATA

  LCD_BAUD = 19_200
  DEGUB_BAUD = 115_200
  BIG_JUMP = $10
  LITTLE_JUMP = $2
  DOWN_LEVEL_TIME = 2000
  UP_LEVEL_TIME = 4000
  RETURN_TO_PULSE = 20_000
  'DEFAULT_STRIP_PAUSE = SHORT_PAUSE
  DEBOUNCE_TIME = 300
  NORDIC_DEBOUNCE_TIME = DEBOUNCE_TIME '400
  INCREASE_CHANGE_TIME = 1000
  NORDIC_DOUBLE_AS_HOLD_INT = 500
  INCREASED_CHANGE_INTERVAL = 500 '1000 '
  SEEK_TEMP_TIME = 2000
  EXTREME_BEEP_INTERVAL = 2000
  FAST_PRESS_THRESHOLD = 3
  ASCII_SPACE = 32
  ASCII_CR = 13

CON ' LCD Codes and Locations
 
  LCD_LINE_0 = 128
  LCD_LINE_1 = 148
  LCD_LINE_2 = 168
  LCD_LINE_3 = 188
  LCD_BACKLIGHT_ON = 17
  LCD_BACKLIGHT_OFF = 18
  LCD_ON_NO_CURSOR = 22
  
  CLOCK_LOC = 0 'LCD_LINE_0 + 10
  DATE_LOC = 0 'LCD_LINE_0 
  RADIO_VOLUME_LOC = LCD_LINE_3 + 13
  RADIO_CHANNEL_LOC = LCD_LINE_2 + 12
  RADIO_RSSI_LOC = LCD_LINE_3 + 5

  STRIP_DISPLAY_TYPE_LOC = LCD_LINE_2 
  STRIP_BRIGHTNESS_LOC = LCD_LINE_2 + 13
  STRIP_SPEED_LOC = LCD_LINE_3 + 15
  STRIP_COLOR_LOC = LCD_LINE_2 + 9
  STRIP_SIZE_LOC = LCD_LINE_3 + 8
  LARSEN_PATTERNS_LOC = LCD_LINE_3 + 12
  LARSEN_SIZE_LOC = LCD_LINE_2 + 15
  LARSEN_BRIGHTNESS_LOC = STRIP_BRIGHTNESS_LOC
  LARSEN_SPEED_LOC = STRIP_SPEED_LOC
  XMAS_BRIGHTNESS_LOC = LCD_LINE_2 + 13
  XMAS_DELAY_LOC = STRIP_SPEED_LOC
  'XMAS_COLOR_LOC = LCD_LINE_2 + 13
  XMAS_SIZE_LOC = STRIP_SIZE_LOC
  XMAS_TEXT_LOC = LCD_LINE_2
  BACKLIGHT_LOC = LCD_LINE_2 + 13
  ADJUST_YEAR_LOC = LCD_LINE_2 + 8
  ADJUST_MONTH_LOC = LCD_LINE_3 + 9
  ADJUST_DATE_LOC = LCD_LINE_2 + 7
  ADJUST_DAY_LOC = LCD_LINE_3 + 6
  ADJUST_MINUTE_LOC = LCD_LINE_2 + 9
  ADJUST_SECOND_LOC = LCD_LINE_3 + 9
  ADJUST_HOUR_LOC = LCD_LINE_2 + 7
  ADJUST_TIME_MODE_LOC = LCD_LINE_3 + 11
  FOREGROUND_COLOR_LOC = LCD_LINE_2 + 7 
  BACKGROUND_COLOR_LOC = LCD_LINE_3 + 7 
  
  DEFAULT_TEMP_LOC = LCD_LINE_0

CON

  FOB_PAYLOAD_SIZE = 4

  'parameter offset enumeration
  #0, MIN_OFFSET, MAX_OFFSET, SMALL_CHANGE_OFFSET, BIG_CHANGE_OFFSET
  
  'adjustMode enumeration
  #0, ADJUST_RADIO, ADJUST_STRIP_DISPLAY_TYPE, ADJUST_STRIP_BRIGHTNESS_SPEED, {
    } ADJUST_STRIP_COLOR, ADJUST_STRIP_SIZE, {
    } {ADJUST_XMAS_BRIGHTNESS_SPEED, ADJUST_XMAS_COLOR_SIZE, ADJUST_XMAS_TEXT,} {
    } ADJUST_BACKLIGHT, ADJUST_YEAR_MONTH, ADJUST_DATE_DAY, ADJUST_HOUR_MODE, {
    } ADJUST_MINUTE_SECOND, ADJUST_OTHER

  HIGHEST_ADJUST_MODE = ADJUST_BACKLIGHT '**ADJUST_OTHER
  
  ' subMode enumeration
  #0, RADIO_DUAL, RADIO_FINE_VOLUME, RADIO_FAVORITES
  #0, STRIP_DUAL, STRIP_FINE_BRIGHTNESS, STRIP_MODE, STRIP_SIZE
  #0, XMAS_DUAL, XMAS_FINE_BRIGHTNESS
  
  '#0, OTHER_PULSE
  '#0, ADJUST_PULSE_BRIGHTNESS, ADJUST_PULSE_ON, ADJUST_PULSE_OFF, ADJUST_BLINK_ON, ADJUST_BLINK_OFF, ADJUST_BLINK_PAUSE

  ' level enumeration
  #0, TOP_LEVEL, SUB_LEVEL, SUB_SUB_LEVEL

  #0, NO_INVERT, YES_INVERT
  #0, NO_ROLL_OVER, YES_ROLL_OVER
  ' changeMode enumeration
  #0, NORMAL_CHANGE, FASTER_CHANGE

  #0, OFF, ON, ON_LONG

  #0, AM_PM_TIME, MILITARY_TIME

  ' changableTimeFlag
  #0, NO_ADJUST, YEAR_MONTH_ADJUST, DATE_DAY_ADJUST, HOUR_MODE_ADJUST, MINUTE_SECOND_ADJUST

  #0, YEAR_INDEX, MONTH_INDEX, DATE_INDEX, DAY_INDEX, HOUR_INDEX, MINUTE_INDEX, SECOND_INDEX

  TIME_ARRAY_SIZE = SECOND_INDEX + 1

  #0, SPARKFUN_STRAND, CHINA_STAND

  STRAND_TYPE = CHINA_STAND

  ' buttonType enumeration
  #0, KEYPAD_BUTTON, NORDIC_BUTTON

  ' displayType enumeration
  #0, PATTERN_DISPLAY, LARSEN_DISPLAY, COLOR_DROP_DISPLAY, OFF_DISPLAY

  'setClockFlag enumeration
  #0, NO_SET_CLOCK, YES_SET_CLOCK
  
  'setOtherFlag enumeration
  #0, NO_SET_OTHER, YES_SET_OTHER
    
OBJ

  Leds : "jm_ws2801"                               ' uses one cog
  Pst : "Parallax Serial Terminal"                 ' uses one cog
  Lcd : "Parallax Serial Terminal"                 ' uses one cog 
  Keypad : "WearableKeypad130430b"                 ' uses one cog
  Radio : "RDA5807SS_Driver2"                      ' Not used yet
  Eeprom : "Store2Eeprom"                          ' Not used yet
  Beep : "Piezo_130514a"
  Clock : "DS1307_RTCEngine"                       ' Not used yet
  Nordic : "NordicBeta110806a"
  Adc : "MCP3002_110808a"                          ' Not used yet
  
VAR   

  'long spokeBuffer[LEDS_PER_SPOKE]
  'long padding0[100]
  long stripOutputBuffer[MAX_BUFFER_SIZE]      
  long rawColorsBuffer[MAX_BUFFER_SIZE]  
  long padding1[100]
  'long rotationBuffer[MAX_BUFFER_SIZE]
  'long patternBuffer[LARGEST_POSSIBLE_PATTERN + 1]
  long expandedPattern[LARGEST_POSSIBLE_PATTERN + 1]
  'long stripOutputBuffer1[MAX_BUFFER_SIZE]                                         
  'long cbuf2[MAX_BUFFER_SIZE]
  long patternSize
  long stack1[100]                                              ' for background cogs
  long stack2[16]
  long shortPause, previousButton, button
  long adjustMode, waitTime, downLevelWait, upLevelWait
  long returnToPulseWait, previousAdjustMode
  long lastPressTime, centerPressStart, level
  
  long radioChannel
  long portionIndex, colorCount
  long debounceInterval[2], increaseChangeTime, increasedChangeInterval[2]
  long newButtonTime, changeMode, debounceTimer
  long LcdMode, rssi
  long Frequency                                 ' Curren Frequency
  long i2cLockId
  
  long previousTime[TIME_ARRAY_SIZE], editedTime[TIME_ARRAY_SIZE]
  'long pYear, pMonth, pDate, pDay, pHour, pMinute, pSecond
  long previousSecond, hourMode, timeLocation, dateLocation
  long halfSecond, lastTimeCheck, startTempTime, activeTempFlag
  long activeTempTime, tempStringLocation, tempStringPtr
  long checkTimeInterval, tempStringSize
  long changableTimeFlag, previouschangableTimeFlag
  long previousLoop, padLedMode
  long horExtremeTimer, vertExtremeTimer
  long extremeBeepInterval
  long activeBufferSize, patternsInBuffer
  long debugLock, stripRotationTimer, stripRotationInterval
  long stripDelay, nordicCog
  long previousFobPressCount, fobPressCount
  long nordicNewButtonTimer, nordicButtonInterval
  long fastPressCount, larsenFade[MAX_LARSEN_SIZE]
  long larsenDirection, larsenIndex, larsenTimer, larsenInterval
  long larsenDelay, activeFore, activeBack
  long pLarsenForeIndex, pLarsenBackIndex, pLarsenSize, pLarsensInUse
  long freezeLarsenForeIndex, freezeLarsenBackIndex, freezeLarsenSize, freezeLarsenInUse
    
  ' MonitorLed variables (used to be local)
  long pDisplayType, pStripBrightness, pStripSize, pStripColor
  long freezeDisplayType, freezeStripSize, freezeStripBrightness
  long freezeStripColor
  long ledPointer ' was result
  ' end MonitorLed variables
                
  '' Call  FM.PassInfo(@ConfigRegs,@Status), the program will copy the data to RAM from RDA5807SS Driver Object
  word ConfigRegs[5]                             ' Registers $02, $03, $04, $05, $06  // $06 reserve for future use
  word Status[4]                                 ' Registers $0A, $0B, $0C, $0D      //  $0C,$0D reserve for future use
  word Stations[32] ' Store the Stations to EEPROM(10-bit Channel information, need to coverte to Frequency)32 stations in total   
    
  byte colorBytes[3]
  byte ptrChnnl                                  ' Point to Current Channel
  byte timeChangedFlag[TIME_ARRAY_SIZE]
  byte buttonRepeatFlag, vertParameterExtremeFlag, horParameterExtremeFlag
  byte pVertParameterExtremeFlag, pHorParameterExtremeFlag
  'byte stripRotateFlag
  byte globalPayload[FOB_PAYLOAD_SIZE + 5] ' Add 1 so a terminating zero may be added if desired. 
  byte buttonType, setClockFlag, setOtherFlag
  
PUB Main

  debugLock := locknew
  repeat until not lockset(debugLock)
  lockclr(debugLock)
  Leds.start(SD, CK, CHANNELS, 5, false)                         ' start WS2801 driver (swap rb)
  shortPause := clkfreq / 10
  halfSecond := clkfreq / 2
  checkTimeInterval := halfSecond
  adjustMode := ADJUST_STRIP_DISPLAY_TYPE 'ADJUST_STRIP_BRIGHTNESS_SPEED
  previousAdjustMode := -1
  previousSecond := -1
  blinkNumber := adjustMode + 1
  'ms := clkfreq / 1000
  downLevelWait := MS_001 * DOWN_LEVEL_TIME
  upLevelWait := MS_001 * UP_LEVEL_TIME
  returnToPulseWait := MS_001 * RETURN_TO_PULSE
  debounceInterval[KEYPAD_BUTTON] := MS_001 * DEBOUNCE_TIME 
  debounceInterval[NORDIC_BUTTON] := MS_001 * NORDIC_DEBOUNCE_TIME 
  increaseChangeTime := MS_001 * INCREASE_CHANGE_TIME
  increasedChangeInterval[KEYPAD_BUTTON] := MS_001 * INCREASED_CHANGE_INTERVAL
  increasedChangeInterval[NORDIC_BUTTON] := MS_001 * INCREASED_CHANGE_INTERVAL
  extremeBeepInterval := MS_001 * EXTREME_BEEP_INTERVAL
  changeMode := NORMAL_CHANGE
  setClockFlag := NO_SET_CLOCK
  setOtherFlag := NO_SET_OTHER
  level := TOP_LEVEL
  previousAdjustMode := -1
  'stripSize := CHANNELS
  activeBufferSize := CHANNELS
  {*******
  i2cLockId := locknew
  repeat until not lockset(i2cLockId)
  Radio.Initialize(RADIO_DATA, RADIO_CLOCK)
  Beep.Init(PIEZO_PIN)

  Radio.BandSet(Radio#US_Eu)
  lockclr(i2cLockId)
  if I2C_CLOCK <> -1
    Clock.rtcEngineStart(I2C_DATA, I2C_CLOCK, i2cLockID)
    Pst.str(String(ASCII_CR, "Real Time Clock Started"))
    waitcnt(clkfreq / 10 + cnt)
    Clock.readTime
    year := Clock.clockYear
    month := Clock.clockMonth
    date := Clock.clockDate
    day := Clock.clockDay
    hour := Clock.clockHour
    minute := Clock.clockMinute
    second := Clock.clockSecond 
    previousSecond := second
    'DisplayTime
  longmove(@editedTime, @lastReadTime, TIME_ARRAY_SIZE)
  ******}
  '**
  Radio.Initialize(RADIO_DATA, RADIO_CLOCK)
  Beep.Init(PIEZO_PIN)
  Radio.BandSet(Radio#US_Eu)
  i2cLockId := locknew
  if I2C_CLOCK <> -1
    Clock.rtcEngineStart(I2C_DATA, I2C_CLOCK, i2cLockID)
    Pst.str(String(ASCII_CR, "Real Time Clock Started"))
    DisplayTime
  '**
 
  
  Keypad.Start(LED_PIN, PAD_PIN_52, PAD_PIN_53, PAD_PIN_51, @blinkBrightness)
  waitcnt(clkfreq / 10 + cnt) 
  Pst.Start(DEGUB_BAUD)   
  Lcd.startRxTx(LCD_RX_PIN, LCD_TX_PIN, 0, LCD_BAUD)
  waitcnt(clkfreq / 10 + cnt)
  
  result := (||stripSpeed) - 1
  stripDelay := speedToDelay[result]
  stripRotationInterval := MS_001 * stripDelay
  stripRotationTimer := cnt
   
  Pst.Str(string(ASCII_CR, "stripDelay = "))
  Pst.Dec(stripDelay)

  result := (||larsenSpeed) - 1
  larsenDelay := speedToDelay[result]
  larsenInterval := MS_001 * larsenDelay
  larsenTimer := cnt
   
  Pst.Str(string(ASCII_CR, "larsenDelay = "))
  Pst.Dec(larsenDelay)

  
  ClearLcd
  LcdMode := ON
  Lcd.Char(LCD_BACKLIGHT_ON)
  Lcd.Char(LCD_ON_NO_CURSOR)
  Pst.Str(string(ASCII_CR, "programName = "))
  Pst.Str(@programName)  
  
  Lcd.Str(string("programName = ", 13))
  Lcd.Str(@programName)

  {repeat result from 1 to 8
    longfill(@stripOutputBuffer, colorBlack[result], CHANNELS)
    SendToStripOutput(@stripOutputBuffer)
    Pause(250)  
      
  LoadPatternToDefaultBuffer(@rainbowPattern)
  waitcnt(clkfreq * 2 + cnt)
  ExpandPattern(@rainbowPattern, 1, true)
  waitcnt(clkfreq * 2 + cnt)
  ExpandPattern(@rainbowPattern, 2, true)
  waitcnt(clkfreq * 2 + cnt)
  ExpandPattern(@rainbowPattern, 3, false)
  waitcnt(clkfreq * 2 + cnt)
  ExpandPattern(@rainbowPattern, 4, true)
  waitcnt(clkfreq * 2 + cnt)  }
  'stripSize := 2
  stripColor := RAINBOW_STRIP
  'BuildLongColors 
  'ExpandPattern(@rainbowPattern, stripSize, true)
  'waitcnt(clkfreq * 2 + cnt)

    
  
  nordicCog := Nordic.Start(NORDIC_SCK, NORDIC_MISO, NORDIC_MOSI, NORDIC_CSN, NORDIC_CE, @config)
  'nordicCog := Nordic.Start(NORDIC_SCK, 0, 1, NORDIC_CSN, NORDIC_CE, @config)
 
  'rxPayload
  Pst.str(String(ASCII_CR, "Nordic Started", ASCII_CR))
  Pst.str(String("Receiver configured.", 13))
  Pst.str(String(13, "Transceiver configured for a four byte payload."))
  
  result := cognew(MonitorLeds, @stack1)

  repeat until not lockset(debugLock)
  Pst.Str(string(ASCII_CR, "LED cog = "))
  Pst.Dec(result)
  lockclr(debugLock)
  repeat until not lockset(debugLock)
  Nordic.SpiWritePub(Nordic#RX_ADDR_P0, @rxAddress, 5)
  Pst.str(String(13, "Reading rx address from module: $"))
  Nordic.SpiReadPub(Nordic#RX_ADDR_P0, @globalPayload, 5)
  repeat result from 0 to 4
    Pst.hex(globalPayload[result], 2)
  SpiCheck
  Nordic.SpiWritePub(Nordic#RX_ADDR_P0, @rxAddress, 5)
  repeat result from 0 to 4
    Pst.hex(globalPayload[result], 2)
  Pst.str(String(13, "Writing to and reading rx address from module: $"))
  Nordic.SpiReadPub(Nordic#RX_ADDR_P0, @globalPayload, 5)
  repeat result from 0 to 4
    Pst.hex(globalPayload[result], 2)
  lockclr(debugLock)
  'waitcnt(clkfreq * 2 + cnt)      
  MainLoop

PUB MainLoop '| tempPause, deltaPause, 

  'deltaPause := ROTATE_PAUSE / 16
  previousButton := Keypad#NO_BUTTON
  newButtonTime := cnt
  lastPressTime := cnt
  lastTimeCheck := cnt
  repeat
    'waitTime += shortPause
    'waitcnt(waitTime) 
    'previousButton := button
    {if cnt - previousLoop < debounceInterval[]
      next
    else
      previousLoop := cnt  }
    result := ina[NORDIC_IRQ]
    if result == 0
      button := ReceivePayload
      buttonType := NORDIC_BUTTON
      'nordicNewButtonTimer := cnt
      
    else
      button := Keypad.CheckButtons
                                  ' This should be the only place in
                                  ' the program where this method is
                                  ' called.
      if button
        buttonType := KEYPAD_BUTTON                          
    
    'Pst.Str(string(ASCII_CR, "button = "))
    'Pst.Dec(button)
    CheckForModeChange
    
    if adjustMode <> previousAdjustMode
      {**if AnyTimeChanged 'timeChangedFlag == 1
        ChangeTime **}
      DebugNewMode
      vertParameterExtremeFlag := 0
      horParameterExtremeFlag := 0
    '**CheckTime

    if button and buttonRepeatFlag <> ON and button <> Keypad#CENTER_BUTTON
      case adjustMode
        ADJUST_RADIO:
          AdjustRadio
        ADJUST_STRIP_DISPLAY_TYPE:
          AdjustDisplayType
        ADJUST_STRIP_BRIGHTNESS_SPEED:
          AdjustStripBrightnessSpeed
        ADJUST_STRIP_COLOR:
          case displayType
            PATTERN_DISPLAY:
              AdjustStripColorSize
            LARSEN_DISPLAY:
              AdjustStripLarsenColors
            COLOR_DROP_DISPLAY..OFF_DISPLAY:
              adjustMode := ADJUST_BACKLIGHT '***
              AdjustBacklight 
        ADJUST_STRIP_SIZE:
          case displayType
            PATTERN_DISPLAY:
              adjustMode := ADJUST_BACKLIGHT   '***
              AdjustBacklight
            LARSEN_DISPLAY:
              AdjustStripLarsenSizes
            COLOR_DROP_DISPLAY..OFF_DISPLAY:
              adjustMode := ADJUST_BACKLIGHT
              AdjustBacklight 
           
        {ADJUST_XMAS_BRIGHTNESS_SPEED:
          AdjustXmasBrightnessSpeed
        ADJUST_XMAS_COLOR_SIZE:
          AdjustXmasColorSize
        ADJUST_XMAS_TEXT:
          AdjustXmasText }
        ADJUST_BACKLIGHT:
          AdjustBacklight
        ADJUST_YEAR_MONTH:
          AdjustYearMonth
        ADJUST_DATE_DAY:
          AdjustDateDay
        ADJUST_HOUR_MODE:
          AdjustHourMode
        ADJUST_MINUTE_SECOND:
          AdjustMinuteSecond
        ADJUST_OTHER:
          AdjustOther
      
    previousButton := button
    previousAdjustMode := adjustMode
 
PUB CheckForModeChange | previousBlinkNumber, previousPadLedMode
' ** add check for double click
  
  'bouncingButtonFlag := 0
  if horParameterExtremeFlag == 1 and pHorParameterExtremeFlag == 0
    horExtremeTimer := cnt
    pHorParameterExtremeFlag := horParameterExtremeFlag
  elseif horParameterExtremeFlag == 1 and cnt - horExtremeTimer > extremeBeepInterval
    pHorParameterExtremeFlag := horParameterExtremeFlag
    horParameterExtremeFlag := 0
  else
    pHorParameterExtremeFlag := horParameterExtremeFlag   
  if horParameterExtremeFlag == 1 and pVertParameterExtremeFlag == 0
    horExtremeTimer := cnt
    pVertParameterExtremeFlag := horParameterExtremeFlag
  elseif vertParameterExtremeFlag == 1 and cnt - vertExtremeTimer > extremeBeepInterval
    pVertParameterExtremeFlag := vertParameterExtremeFlag
    vertParameterExtremeFlag := 0
  else
    pVertParameterExtremeFlag := vertParameterExtremeFlag   

  if buttonType == NORDIC_BUTTON
    result := CheckForModeChangeNordic
    repeat until not lockset(debugLock)
    'Pst.Str(string(ASCII_CR, "Returned from CheckForModeChangeNordic"))
    Pst.Str(string(ASCII_CR, "r= "))
    Pst.Dec(result) 
    Pst.Str(string(ASCII_CR, "b= "))
    Pst.Dec(button) 
    lockclr(debugLock)
    if result == -1
      return
    
  elseif button <> previousButton    
    newButtonTime := cnt
    debounceTimer := cnt
    changeMode := NORMAL_CHANGE
    buttonRepeatFlag := OFF
  elseif cnt - debounceTimer < debounceInterval
    buttonRepeatFlag := ON
    'Pst.Char(".")
    'bouncingButtonFlag := 1
    return
  elseif cnt - newButtonTime > increaseChangeTime and changeMode == NORMAL_CHANGE    
    changeMode := FASTER_CHANGE
    buttonRepeatFlag := ON_LONG
    newButtonTime := cnt
    
  elseif cnt - newButtonTime > increasedChangeInterval[buttonType] and changeMode == FASTER_CHANGE   
    newButtonTime := cnt
    buttonRepeatFlag := ON_LONG
  else  
    buttonRepeatFlag := ON

   { parameterChangeTimer := cnt
  elseif changeMode == FASTER_CHANGE
    if cnt - parameterChangeTimer > increaseChangeTime
      parameterChangeTimer += increaseChangeTime
      if changeMode := FASTER_CHANGE
    buttonRepeatFlag := ON_LONG    }
    
  ' prevent debounceTimer roll over  ** awkward
  debounceTimer := cnt + debounceInterval

  ' Check which LED mode the keypad's LED should be in.
  ' This isn't working correctly. The problem may be in
  ' child object.
  if button <> Keypad#NO_BUTTON and buttonType == KEYPAD_BUTTON 
    lastPressTime := cnt
    previousPadLedMode := padLedMode
    padLedMode := Keypad#BLINK_LED
    if padLedMode <> previousPadLedMode or previousBlinkNumber <> blinkNumber
      repeat until not lockset(debugLock)
      Pst.Str(string(ASCII_CR, "padLedMode := Keypad#BLINK_LED"))
      Pst.Str(string(ASCII_CR, "blinkNumber := Keypad#BLINK_LED"))
      lockclr(debugLock) 
  elseif (padLedMode == Keypad#BLINK_LED and cnt - lastPressTime > returnToPulseWait) or {
       } buttonType == NORDIC_BUTTON 
    padLedMode := Keypad#PULSE_LED
    repeat until not lockset(debugLock)
    Pst.Str(string(ASCII_CR, "padLedMode := Keypad#PULSE_LED"))
    lockclr(debugLock)
    if buttonType <> NORDIC_BUTTON
      return

  repeat until not lockset(debugLock)
  Pst.Str(string(ASCII_CR, "adjustMode := "))
  Pst.Dec(adjustMode)
  if button == Keypad#CENTER_BUTTON and previousButton <> Keypad#CENTER_BUTTON
    centerPressStart := cnt
    Beep.BeeDeepShort
    Pst.Char("^")
    if buttonType == NORDIC_BUTTON
      if fastPressCount < FAST_PRESS_THRESHOLD and adjustMode < HIGHEST_ADJUST_MODE
        adjustMode++
      else
        adjustMode := ADJUST_STRIP_DISPLAY_TYPE '**ADJUST_RADIO
  elseif button == Keypad#NO_BUTTON and previousButton == Keypad#CENTER_BUTTON
    if cnt - centerPressStart > downLevelWait or adjustMode == HIGHEST_ADJUST_MODE
      adjustMode := ADJUST_STRIP_DISPLAY_TYPE '**ADJUST_RADIO
      Pst.Char("#")
    else
      Pst.Char("@")
      if displayType == OFF_DISPLAY     '**
        if adjustMode == ADJUST_STRIP_DISPLAY_TYPE
          adjustMode := ADJUST_BACKLIGHT
          Pst.Char("b")
        else
          adjustMode := ADJUST_STRIP_DISPLAY_TYPE    '**
          Pst.Char("c")
      else    
        adjustMode++
        Pst.Char("+")
  blinkNumber := adjustMode + 1
  Pst.Str(string(ASCII_CR, "_adjustMode := "))
  Pst.Dec(adjustMode)
  lockclr(debugLock)
  
PUB CheckForModeChangeNordic '| localPreviousButton, localButton
' ** add check for double click
' We want to ignore "NO_BUTTON" buttons if they occur within   
' increasedChangeInterval[buttonType] since the last
' Nordic button was pressed.

  repeat until not lockset(debugLock)
  'Pst.Str(string(ASCII_CR, "padLedMode := Keypad#PULSE_LED"))
  lockclr(debugLock)
  
  if button == Keypad#NO_BUTTON and cnt - newButtonTime > increasedChangeInterval[buttonType]
    ' Time limit of treating secong button press as a button which has been held down
    ' expired. Return to normal button checking.
    buttonType := KEYPAD_BUTTON
    changeMode := NORMAL_CHANGE
    buttonRepeatFlag := OFF
    fastPressCount := 0
    repeat until not lockset(debugLock)
    Pst.Str(string(ASCII_CR, "expired "))
    Pst.Dec((cnt - newButtonTime) / MS_001)
    Pst.Char("<") 
    Pst.Dec(fastPressCount)  
    Pst.Char(">") 
    lockclr(debugLock)
  
    return -1
  elseif button == Keypad#NO_BUTTON
    ' No button press but still within time limit to
    ' to treat no press as a repeated button.
    buttonRepeatFlag := ON
    'localButton := button  
    button := previousButton
    return -1
  elseif button <> previousButton
    ' A pressed button which is new.
    newButtonTime := cnt
    debounceTimer := cnt
    nordicNewButtonTimer := cnt
    changeMode := NORMAL_CHANGE
    buttonRepeatFlag := OFF
    fastPressCount := 0
    repeat until not lockset(debugLock)
    Pst.Str(string(ASCII_CR, "new"))
    lockclr(debugLock)
  
  elseif cnt - debounceTimer < debounceInterval[buttonType] and {
       } fastPressCount < FAST_PRESS_THRESHOLD
    ' Repeated button too soon to treat as an intentional press.
    buttonRepeatFlag := ON
    newButtonTime := cnt
    fastPressCount++
    ''watchForFastNordicButtons := 1
    ''nordicNewButtonTimer := cnt
    'Pst.Char(".")
    'bouncingButtonFlag := 1
    repeat until not lockset(debugLock)
    Pst.Char(".")
    Pst.Dec((cnt - debounceTimer) / MS_001)  
    Pst.Char(".")
    Pst.Dec(fastPressCount)  
    Pst.Char(".")
    lockclr(debugLock)
  
    return -1

    ' At this point the button should be considered an intentional
    ' quick second press indicating holding the button down.  
  else'if cnt - newButtonTime < increasedChangeInterval[buttonType] 'and changeMode == NORMAL_CHANGE    
    changeMode := FASTER_CHANGE
    buttonRepeatFlag := ON_LONG
    newButtonTime := cnt
    fastPressCount++
    repeat until not lockset(debugLock)
    Pst.Str(string(ASCII_CR, "held |"))
    Pst.Dec(fastPressCount)  
    Pst.Char("|")
    lockclr(debugLock)
  
PUB AnyTimeChanged | localIndex

  repeat localIndex from YEAR_INDEX to SECOND_INDEX
    result += timeChangedFlag[localIndex]
    if timeChangedFlag[localIndex]
      repeat until not lockset(debugLock)
      Pst.Str(string(ASCII_CR, "timeChangedFlag["))
      Pst.Dec(localIndex)
      Pst.Str(string("] = ")) 
      Pst.Dec(timeChangedFlag[localIndex])
      lockclr(debugLock)
      
  if result
    repeat until not lockset(debugLock)
    Pst.Str(string(ASCII_CR, "result = "))
    Pst.Dec(result)
    lockclr(debugLock)
    
PRI ReceivePayload | localIndex

  Nordic.ReadPayload(@globalPayload)
  Pst.str(String(13, "Received: %"))
  Pst.Bin(globalPayload[0], 5)
  Pst.str(String(13, "Received:"))
  repeat result from 0 to 4
    Pst.str(String(" $"))  
    Pst.hex(globalPayload[result], 2)
  fobPressCount := (globalPayload[1] << 8) + globalPayload[2]
  Pst.str(String(13, "fobPressCount = "))
  Pst.dec(fobPressCount)
  result := (fobPressCount - previousFobPressCount) - 1

  previousFobPressCount := fobPressCount
  
  if result
    Pst.str(String(13, "Missiing Fob Transmissions = "))
    Pst.dec(result)
  else
    Pst.str(String(13, "No Missing Fob Transmissions"))

  ' if multiple button presses are to be detected, they should be
  ' check prior to checking for individual buttons.
  ' Only the lowest numbered key press is detected.  
  repeat localIndex from Keypad#CENTER_BUTTON to Keypad#RIGHT_BUTTON
    Pst.str(String(13, "localIndex = "))
    Pst.dec(localIndex)
    Pst.str(String(13, "Received: %"))
    Pst.Bin(globalPayload[0], 8)
    Pst.str(String(13, "fobButtonMask["))
    Pst.dec(localIndex)
    Pst.str(String("] = %"))
    Pst.Bin(fobButtonMask[localIndex], 8)
  
    if (globalPayload[0] & fobButtonMask[localIndex]) <> fobButtonMask[localIndex]
      result := localIndex
      Pst.str(String(13, "Match Found")) 
      quit
  Pst.str(String(13, "result = "))
  Pst.dec(result) 
            
PUB AdjustParameter(parameterPtr, limitsPtr, invertFlag, rollOverFlag) | increase, decrease, increaseSmall, {
                                                         } decreaseSmall

  if invertFlag == NO_INVERT
    increase := Keypad#UP_BUTTON
    decrease := Keypad#DOWN_BUTTON
    increaseSmall := Keypad#RIGHT_BUTTON
    decreaseSmall := Keypad#LEFT_BUTTON
  else
    increase := Keypad#DOWN_BUTTON
    decrease := Keypad#UP_BUTTON 
    increaseSmall := Keypad#LEFT_BUTTON
    decreaseSmall := Keypad#RIGHT_BUTTON

  if button == increase 'and previousButton <> increase
    if long[parameterPtr] =< long[limitsPtr][MAX_OFFSET] - long[limitsPtr][BIG_CHANGE_OFFSET]
      long[parameterPtr] += long[limitsPtr][BIG_CHANGE_OFFSET]
      vertParameterExtremeFlag := 0
    elseif rollOverFlag
      result := long[parameterPtr] + long[limitsPtr][BIG_CHANGE_OFFSET] - 1
      long[parameterPtr] := result - (long[limitsPtr][MAX_OFFSET] - {
                              } long[limitsPtr][MIN_OFFSET])
      Beep.BeeDeep
    else                                
      long[parameterPtr] := long[limitsPtr][MAX_OFFSET]
      if vertParameterExtremeFlag == 0
        Beep.BeepBeep
        vertParameterExtremeFlag := 1                                                             
  elseif button == decrease and previousButton <> decrease
    if long[parameterPtr] => long[limitsPtr][BIG_CHANGE_OFFSET]
      long[parameterPtr] -= long[limitsPtr][BIG_CHANGE_OFFSET]
      vertParameterExtremeFlag := 0
    elseif rollOverFlag
      result := long[parameterPtr] - long[limitsPtr][BIG_CHANGE_OFFSET] + 1
      long[parameterPtr] := result + (long[limitsPtr][MAX_OFFSET] - {
                              } long[limitsPtr][MIN_OFFSET])
      Beep.BeeDeep
    else
      long[parameterPtr] := long[limitsPtr][MIN_OFFSET]
      if vertParameterExtremeFlag == 0
        Beep.BeepBeep
        vertParameterExtremeFlag := 1                                                                         
  elseif button == increaseSmall and previousButton <> increaseSmall
    if long[parameterPtr] =< long[limitsPtr][MAX_OFFSET] - long[limitsPtr][SMALL_CHANGE_OFFSET]
      long[parameterPtr] += long[limitsPtr][SMALL_CHANGE_OFFSET]
      vertParameterExtremeFlag := 0
    elseif rollOverFlag
      result := long[parameterPtr] + long[limitsPtr][SMALL_CHANGE_OFFSET] - 1
      long[parameterPtr] := result - (long[limitsPtr][MAX_OFFSET] - {
                              } long[limitsPtr][MIN_OFFSET])
      Beep.BeeDeep
    else
      long[parameterPtr] := long[limitsPtr][MAX_OFFSET]
      if vertParameterExtremeFlag == 0
        Beep.BeepBeep
        vertParameterExtremeFlag := 1
  elseif button == decreaseSmall and previousButton <> decreaseSmall
    if long[parameterPtr] => long[limitsPtr][SMALL_CHANGE_OFFSET]
      long[parameterPtr] -= long[limitsPtr][SMALL_CHANGE_OFFSET]
      vertParameterExtremeFlag := 0
    elseif rollOverFlag
      result := long[parameterPtr] - long[limitsPtr][SMALL_CHANGE_OFFSET] + 1
      long[parameterPtr] := result + (long[limitsPtr][MAX_OFFSET] - {
                              } long[limitsPtr][MIN_OFFSET])
      Beep.BeeDeep
    else
      long[parameterPtr] := long[limitsPtr][MIN_OFFSET]
      if vertParameterExtremeFlag == 0
        Beep.BeepBeep
        vertParameterExtremeFlag := 1     
    'elseif button == Keypad#CENTER_BUTTON and previousButton <> Keypad#CENTER_BUTTON

PUB AdjustParameters(vertParameterPtr, vertLimitsPtr, vertInvertFlag, vertRollOverFlag, {
                   } horParameterPtr, horLimitsPtr, horInvertFlag, horRollOverFlag) | {
                   } vertIncrease, vertDecrease, horIncrease, horDecrease, {
                   } vertChangeValue, horChangeValue

  if vertInvertFlag == NO_INVERT
    vertIncrease := Keypad#UP_BUTTON
    vertDecrease := Keypad#DOWN_BUTTON
  else
    vertIncrease := Keypad#DOWN_BUTTON
    vertDecrease := Keypad#UP_BUTTON 
  if horInvertFlag == NO_INVERT
    horIncrease := Keypad#RIGHT_BUTTON
    horDecrease := Keypad#LEFT_BUTTON
  else
    horIncrease := Keypad#LEFT_BUTTON
    horDecrease := Keypad#RIGHT_BUTTON

  repeat until not lockset(debugLock)
  Pst.Str(string(ASCII_CR, "long[vertParameterPtr] = "))
  Pst.Dec(long[vertParameterPtr])

  if changeMode == FASTER_CHANGE
    Pst.Str(string(ASCII_CR, "changeMode == FASTER_CHANGE"))
    vertChangeValue := long[vertLimitsPtr][BIG_CHANGE_OFFSET]
    horChangeValue := long[horLimitsPtr][BIG_CHANGE_OFFSET]
  else
    vertChangeValue := long[vertLimitsPtr][SMALL_CHANGE_OFFSET]
    horChangeValue := long[horLimitsPtr][SMALL_CHANGE_OFFSET]
       
  if button == vertIncrease 'and previousButton <> vertIncrease
    Pst.Str(string(ASCII_CR, "button == vertIncrease"))
   
    if long[vertParameterPtr] =< long[vertLimitsPtr][MAX_OFFSET] - vertChangeValue
    'if long[vertParameterPtr] + vertChangeValue =< long[vertLimitsPtr][MAX_OFFSET] '- vertChangeValue
      long[vertParameterPtr] += vertChangeValue
      vertParameterExtremeFlag := 0
    elseif vertRollOverFlag ' roll over assumes parameters can't be negative
      result := long[vertParameterPtr] + vertChangeValue - 1
      long[vertParameterPtr] := result - (long[vertLimitsPtr][MAX_OFFSET] - {
                              } long[vertLimitsPtr][MIN_OFFSET])
      Beep.BeeDeep
    else                                
      long[vertParameterPtr] := long[vertLimitsPtr][MAX_OFFSET]
      if vertParameterExtremeFlag == 0
        Beep.BeepBeep
        vertParameterExtremeFlag := 1                                                            
  elseif button == vertDecrease 'and previousButton <> vertDecrease
    Pst.Str(string(ASCII_CR, "button == vertDecrease"))
    if long[vertParameterPtr] => long[vertLimitsPtr][MIN_OFFSET] + vertChangeValue
      long[vertParameterPtr] -= vertChangeValue
      vertParameterExtremeFlag := 0
    elseif vertRollOverFlag
      result := long[vertParameterPtr] - vertChangeValue + 1
      long[vertParameterPtr] := result + (long[vertLimitsPtr][MAX_OFFSET] - {
                              } long[vertLimitsPtr][MIN_OFFSET])
      Beep.BeeDeep
    else
      long[vertParameterPtr] := long[vertLimitsPtr][MIN_OFFSET]
      if vertParameterExtremeFlag == 0
        Beep.BeepBeep
        vertParameterExtremeFlag := 1
                                                                           
  elseif button == horIncrease 'and previousButton <> horIncrease
    Pst.Str(string(ASCII_CR, "button == horIncrease"))
    if long[horParameterPtr] =< long[horLimitsPtr][MAX_OFFSET] - horChangeValue
      long[horParameterPtr] += horChangeValue
      horParameterExtremeFlag := 0
    elseif horRollOverFlag
      result := long[horParameterPtr] + horChangeValue - 1
      long[horParameterPtr] := result - (long[horLimitsPtr][MAX_OFFSET] - {
                              } long[horLimitsPtr][MIN_OFFSET])
      Beep.BeeDeep
    else
      long[horParameterPtr] := long[horLimitsPtr][MAX_OFFSET]
      if horParameterExtremeFlag == 0
        Beep.BeepBeep
        horParameterExtremeFlag := 1
  elseif button == horDecrease 'and previousButton <> horDecrease
    Pst.Str(string(ASCII_CR, "button == horDecrease"))
    if long[horParameterPtr] => long[horLimitsPtr][MIN_OFFSET] + horChangeValue
      long[horParameterPtr] -= horChangeValue
      Pst.Str(string(ASCII_CR, "long[horParameterPtr] -= horChangeValue")) 
    elseif horRollOverFlag
      result := long[vertParameterPtr] - horChangeValue + 1
      long[horParameterPtr] := result + (long[horLimitsPtr][MAX_OFFSET] - {
                              } long[horLimitsPtr][MIN_OFFSET])
      Beep.BeeDeep
      Pst.Str(string(ASCII_CR, "horRollOverFlag")) 
    else
      long[horParameterPtr] := long[horLimitsPtr][MIN_OFFSET]
      Pst.Str(string(ASCII_CR, "long[horParameterPtr] := long[horLimitsPtr][MIN_OFFSET]")) 
      if horParameterExtremeFlag == 0
        Beep.BeepBeep
        horParameterExtremeFlag := 1       
    'elseif button == Keypad#CENTER_BUTTON and previousButton <> Keypad#CENTER_BUTTON

  Pst.Str(string(ASCII_CR, "long[vertParameterPtr] = "))
  Pst.Dec(long[vertParameterPtr])
  Pst.Str(string(ASCII_CR, "long[horParameterPtr] = "))
  Pst.Dec(long[horParameterPtr])
  lockclr(debugLock)
  
PUB DebugNewMode                                                                                    

  repeat until not lockset(debugLock)
  Pst.Str(string(ASCII_CR, "New Adjustment Mode: "))
  lockclr(debugLock)
  ClearLcd
  Lcd.Char(LCD_LINE_0)
  DisplayDisplayType(displayType)
  'DisplayClock(CLOCK_LOC)

{ADJUST_RADIO, ADJUST_STRIP_DISPLAY_TYPE, ADJUST_STRIP_BRIGHTNESS_SPEED, {
    } ADJUST_STRIP_COLOR, ADJUST_STRIP_SIZE}  
  case adjustMode
    ADJUST_RADIO:
      timeLocation := CLOCK_LOC
      dateLocation := DATE_LOC
      repeat until not lockset(i2cLockId)    

      
      radioChannel := Radio.CheckChannel
      radioVolume := Radio.CheckVolume
      repeat until not lockset(debugLock)
      Pst.Str(string(ASCII_CR, "Radio Mode"))      
      '                         1
      '               01234567890123456789
      Lcd.Char(LCD_LINE_1)
      Lcd.Str(string("     Radio Mode"))
      'BothStr(string(ASCII_CR, "Channel = "))    
      'Lcd.Char(RADIO_CHANNEL_LOC)
      'BothDec(radioChannel, 6, ASCII_SPACE)
      result := Radio.CheckFreq
      Pst.Str(string(ASCII_CR, "frequency = "))
      'Pst.Dec(result)
      Lcd.Char(RADIO_CHANNEL_LOC)
      BothDec(result, 6, ASCII_SPACE)
  
      BothStr(string(ASCII_CR, "RSSI: "))
      rssi := Radio.CheckRSSI
      Lcd.Char(RADIO_RSSI_LOC)
      BothDec(rssi, 3, ASCII_SPACE)
      BothStr(string("Vol= "))
      Lcd.Char(RADIO_VOLUME_LOC)
      BothDec(radioVolume, 2, ASCII_SPACE)
      lockclr(debugLock)
      lockclr(i2cLockId)

      
    ADJUST_STRIP_DISPLAY_TYPE: 
      repeat until not lockset(debugLock)
      timeLocation := CLOCK_LOC
      dateLocation := DATE_LOC
      Pst.Str(string(ASCII_CR, "LED Display Type", ASCII_CR))
      '                         1
      '               01234567890123456789
      Lcd.Char(LCD_LINE_1)
      Lcd.Str(string("  LED Display Type"))
      
      'Lcd.Char(LCD_LINE_2)
      '               01234567890123456789
      
      Lcd.Char(STRIP_DISPLAY_TYPE_LOC)
      lockclr(debugLock)      
      DisplayDisplayType(displayType)
    
    ADJUST_STRIP_BRIGHTNESS_SPEED:
      timeLocation := CLOCK_LOC
      dateLocation := DATE_LOC
      repeat until not lockset(debugLock)
      Pst.Str(string(ASCII_CR, "LED Brightness/Delay Mode"))
      '                         12:59:59AM
      '                         1  
      '               01234567890123456789
      Lcd.Char(LCD_LINE_1)
      Lcd.Str(string("LED Brightness/Speed"))
      Lcd.Char(LCD_LINE_2)
      '               01234567890123456789
      BothStr(string("Brightness = "))
      Lcd.Char(STRIP_BRIGHTNESS_LOC)
  
      BothDec(stripBrightness, 3, ASCII_SPACE)
      Lcd.Char(LCD_LINE_3)
      '               01234567890123456789
      BothStr(string("Scroll Speed = "))
      Lcd.Char(STRIP_SPEED_LOC)
      'BothDec(stripPause, 5, ASCII_SPACE)
      Pst.Dec(stripSpeed)
      LcdDecClean(stripSpeed, 4)
      lockclr(debugLock)
    
    ADJUST_STRIP_COLOR:
      timeLocation := CLOCK_LOC
      dateLocation := DATE_LOC

      case displayType
        PATTERN_DISPLAY:     
          repeat until not lockset(debugLock)
          Pst.Str(string(ASCII_CR, "adjustMode = ADJUST_STRIP_COLOR")) 
          Pst.Str(string(ASCII_CR, "displayType = PATTERN_DISPLAY", ASCII_CR)) 
          Lcd.Char(LCD_LINE_1)
          '               01234567890123456789
          BothStr(string("Adjust Color & Size"))
          Lcd.Char(LCD_LINE_2)
          BothStr(string(" Color = "))
          Lcd.Char(STRIP_COLOR_LOC)
          lockclr(debugLock)
          DisplayColorSetting(stripColor)
          'BothDec(stripColor, 2, ASCII_SPACE)                                                                                                                                                                                                                                          
          Lcd.Char(LCD_LINE_3)
          '               01234567890123456789
          repeat until not lockset(debugLock)
          BothStr(string(" Size = "))
          Lcd.Char(STRIP_SIZE_LOC)
          BothDec(stripSize, 3, ASCII_SPACE)
          lockclr(debugLock)
        LARSEN_DISPLAY:
          repeat until not lockset(debugLock)
          Pst.Str(string(ASCII_CR, "adjustMode = ADJUST_STRIP_COLOR")) 
          Pst.Str(string(ASCII_CR, "displayType = LARSEN_DISPLAY", ASCII_CR)) 
          '               01234567890123456789
          BothStr(string("Adjust Cylon Colors"))
          Lcd.Char(LCD_LINE_2)
          BothStr(string("Fore = "))
          Lcd.Char(FOREGROUND_COLOR_LOC)
          lockclr(debugLock)
          DisplayColorSetting(larsenForeIndex)
          'BothDec(stripColor, 2, ASCII_SPACE) 
          Lcd.Char(LCD_LINE_3)
          '               01234567890123456789
          repeat until not lockset(debugLock)
          BothStr(string("Back = "))
          Lcd.Char(BACKGROUND_COLOR_LOC)
          lockclr(debugLock)
          DisplayColorSetting(larsenBackIndex)
          
    ADJUST_STRIP_SIZE:
      'repeat until not lockset(debugLock)
      timeLocation := CLOCK_LOC
      dateLocation := DATE_LOC
      {Pst.Str(string(ASCII_CR, "LED Size Mode", ASCII_CR))
      lockclr(debugLock)
      '                         1
      '               01234567890123456789
      Lcd.Char(LCD_LINE_1)
      Lcd.Str(string("LED Color&Size Mode"))   }
      case displayType
        PATTERN_DISPLAY:
          adjustMode := ADJUST_BACKLIGHT
          repeat until not lockset(debugLock)
          Pst.Str(string(ASCII_CR, "Backlight Mode"))
          '                         1
          '               01234567890123456789
          Lcd.Str(string(LCD_LINE_1, "   Backlight Mode"))
          Lcd.Char(LCD_LINE_2)
          BothStr(string("Backlight is "))
          Lcd.Char(BACKLIGHT_LOC)
          case lcdMode
            ON:
              Lcd.Str(@on_text)
            OFF:
              Lcd.Str(@off_text)   
          lockclr(debugLock)
     
          {Lcd.Char(LCD_LINE_2)
          '               01234567890123456789
          repeat until not lockset(debugLock)
          BothStr(string("Pattern Fill = "))
          Lcd.Char(STRIP_SIZE_LOC)
       
          BothDec(stripSize, 3, ASCII_SPACE) 
         
          lockclr(debugLock)   }
        LARSEN_DISPLAY:
          Lcd.Char(LCD_LINE_1)
          repeat until not lockset(debugLock)
          BothStr(string(" Adjust Cylon Sizes"))
          Lcd.Char(LCD_LINE_2)
          '               01234567890123456789
          BothStr(string("Spread Amount= "))
          Lcd.Char(LARSEN_SIZE_LOC)
     
          BothDec(larsenSize, 3, ASCII_SPACE) 
          Lcd.Char(LCD_LINE_3)
          '               01234567890123456789
         
          BothStr(string("Divisions = "))
          Lcd.Char(LARSEN_PATTERNS_LOC)
          BothDec(larsensInUse, 3, ASCII_SPACE)
          lockclr(debugLock)
         
  
    {ADJUST_XMAS_BRIGHTNESS_SPEED:
      timeLocation := CLOCK_LOC
      dateLocation := 0
      Pst.Str(string(ASCII_CR, "Christmas Light Mode"))
      '                         1
      '               01234567890123456789
      Lcd.Char(LCD_LINE_1)
      Lcd.Str(string("Christmas Light Mode"))
      BothStr(string(ASCII_CR, "Brightness = "))
      Lcd.Char(XMAS_BRIGHTNESS_LOC)
      BothDec(xmasBrightness, 3, ASCII_SPACE)
      BothStr(string(ASCII_CR, "Scroll Speed = "))
      Lcd.Char(XMAS_DELAY_LOC)
      BothDec(xmasSpeed, 5, ASCII_SPACE)     
    ADJUST_XMAS_COLOR_SIZE:
      timeLocation := CLOCK_LOC
      dateLocation := 0
      Pst.Str(string(ASCII_CR, "Christmas Light Mode"))
      '                         1
      '               01234567890123456789
      Lcd.Char(LCD_LINE_1)
      Lcd.Str(string("Xmas Color/Size Mode"))
      Lcd.Char(XMAS_SIZE_LOC)
      BothDec(xmasSize, 3, ASCII_SPACE)
      'Lcd.Char()
      
  
    ADJUST_XMAS_TEXT:
      timeLocation := CLOCK_LOC
      dateLocation := 0
      Pst.Str(string(ASCII_CR, "Christmas Light Mode"))
      '                         1
      '               01234567890123456789
      Lcd.Char(LCD_LINE_1)
      Lcd.Str(string("   Christmas Text"))
      Lcd.Char(XMAS_TEXT_LOC)  }
    ADJUST_BACKLIGHT:
      timeLocation := CLOCK_LOC
      dateLocation := DATE_LOC
      repeat until not lockset(debugLock)
      Pst.Str(string(ASCII_CR, "Backlight Mode"))
      '                         1
      '               01234567890123456789
      Lcd.Str(string(LCD_LINE_1, "   Backlight Mode"))
      Lcd.Char(LCD_LINE_2)
      BothStr(string("Backlight is "))
      Lcd.Char(BACKLIGHT_LOC)
      case lcdMode
        ON:
          Lcd.Str(@on_text)
        OFF:
          Lcd.Str(@off_text)   
      lockclr(debugLock)
      'BothDec(LcdMode, 1, ASCII_SPACE)
      'Pst.Dec(LcdMode)
      'Lcd.Dec(LcdMode)
      'adjustMode enumeration
    ADJUST_YEAR_MONTH:
      timeLocation := CLOCK_LOC
      dateLocation := DATE_LOC
      Lcd.Char(LCD_LINE_1)
      Lcd.Str(string(" Adjust Year & Month"))
      Lcd.Char(LCD_LINE_2)
      repeat until not lockset(debugLock)
      BothStr(string("Year = "))
      Lcd.Char(ADJUST_YEAR_LOC)
      BothDec(year, 4, "0")
      Lcd.Char(LCD_LINE_3)
      BothStr(string("Month = "))
      Lcd.Char(ADJUST_MONTH_LOC)
      BothDec(month, 2, "0")
      lockclr(debugLock)
       
    ADJUST_DATE_DAY:
      timeLocation := CLOCK_LOC
      dateLocation := DATE_LOC
      Lcd.Char(LCD_LINE_1)
      Lcd.Str(string("Adjust Date and Day"))
      Lcd.Char(LCD_LINE_2)
      repeat until not lockset(debugLock)
      BothStr(string("Date = "))
      Lcd.Char(ADJUST_DATE_LOC)
      BothDec(date, 2, ASCII_SPACE)
      Lcd.Char(LCD_LINE_3)
      BothStr(string("Day = "))
      Lcd.Char(ADJUST_DAY_LOC)
      'BothDec(day, 1, ASCII_SPACE)
      Pst.Dec(editedTime[DAY_INDEX])
      Lcd.Dec(editedTime[DAY_INDEX])
      lockclr(debugLock)
      
    ADJUST_HOUR_MODE:
      timeLocation := CLOCK_LOC
      dateLocation := 0
      Lcd.Char(LCD_LINE_1)
      Lcd.Str(string("Adjust Hour and Mode"))
      Lcd.Char(LCD_LINE_2)
      repeat until not lockset(debugLock)
      BothStr(string("hour = "))
      Lcd.Char(ADJUST_HOUR_LOC)
      BothDec(hour, 2, ASCII_SPACE)
      Lcd.Char(LCD_LINE_3)
      BothStr(string("hourMode = "))
      Lcd.Char(ADJUST_TIME_MODE_LOC)
      case hourMode         
        AM_PM_TIME:
          BothStr(@am_pm)
        MILITARY_TIME:
          BothStr(@military)
      lockclr(debugLock)
      'BothDec(mode, 1, ASCII_SPACE) 
    ADJUST_MINUTE_SECOND:
      timeLocation := CLOCK_LOC
      dateLocation := 0
      Lcd.Char(LCD_LINE_1)
      Lcd.Str(string("Adjust Minute and Second"))
      Lcd.Char(LCD_LINE_2)
      repeat until not lockset(debugLock)
      BothStr(string("Minute = "))
      Lcd.Char(ADJUST_MINUTE_LOC)
      BothDec(minute, 2, "0")
      Lcd.Char(LCD_LINE_3)
      BothStr(string("Second = "))
      Lcd.Char(ADJUST_SECOND_LOC)
      BothDec(second, 2, "0") 
      lockclr(debugLock)
      
      

  
    ADJUST_OTHER:
      timeLocation := CLOCK_LOC
      dateLocation := DATE_LOC
      repeat until not lockset(debugLock)
      Pst.Str(string(ASCII_CR, "Other Adjustments Mode"))
      lockclr(debugLock)
      '                         1
      '               01234567890123456789
      Lcd.Char(LCD_LINE_1)
      Lcd.Str(string("     Other Mode"))
      'Lcd.Char()

PRI DisplayColorSetting(localColor)

  repeat until not lockset(debugLock)
  case localColor         
    OFF_STRIP:       '01234567890123456789
      BothStr(string("Off      "))
    RED_STRIP:
      BothStr(string("Red      "))
    ORANGE_STRIP_STRIP:
      BothStr(string("Orange   "))                                                                                                                                                                                                                                                                                                                                                                    
    YELLOW_STRIP:    
      BothStr(string("Yellow   "))
    GREEN_STRIP:    
      BothStr(string("Green    "))
    BLUE_STRIP:    
      BothStr(string("Blue     "))
    VIOLET_STRIP:    
      BothStr(string("Violet   "))
    WHITE_STRIP:    
      BothStr(string("White    "))
    INDIGO_STRIP:    
      BothStr(string("Indigo   "))
    RAINBOW_STRIP:
      BothStr(string("Rainbow  ")) 
    other:
      BothStr(string("Unknown?"))
  lockclr(debugLock)
{
PRI DisplayColorSetting(localColor)

  repeat until not lockset(debugLock)
  case localColor         
    OFF_STRIP:       '01234567890123456789
      BothStr(string("        Off         "))
    RED_STRIP:
      BothStr(string("        Red         "))
    ORANGE_STRIP_STRIP:                                                                                                                                                                                                                                                                                                               
      BothStr(string("       Orange       "))                                                                                                                                                                                                                                                                                                                                                                    
    YELLOW_STRIP:    
      BothStr(string("       Yellow       "))
    GREEN_STRIP:    
      BothStr(string("       Green        "))
    BLUE_STRIP:    
      BothStr(string("        Blue        "))
    VIOLET_STRIP:    
      BothStr(string("       Violet       "))
    WHITE_STRIP:    
      BothStr(string("       White        "))
    INDIGO_STRIP:    
      BothStr(string("       Indigo       "))
    RAINBOW_STRIP:
      BothStr(string("      Rainbow       ")) 
    other:
      BothStr(string("     Unknown?      "))
  lockclr(debugLock)
}
PRI DisplayDisplayType(localDisplayType)

  repeat until not lockset(debugLock)
  case localDisplayType
    PATTERN_DISPLAY: '01234567890123456789
      BothStr(string("      Pattern       "))
    LARSEN_DISPLAY:
      BothStr(string("       Cylon        "))
    COLOR_DROP_DISPLAY:
      BothStr(string("     Color Drop     "))
    OFF_DISPLAY:
      BothStr(string("        Off         "))
    other:
      BothStr(string("     Unknown?      "))
  lockclr(debugLock)
       
PUB AdjustRadio | previousVolume, previousSteps

  
  
  if button == Keypad#CENTER_BUTTON
    return

  previousVolume := radioVolume
  previousSteps := radioSteps

  AdjustParameters(@radioVolume, @radioVLimits, NO_INVERT, NO_ROLL_OVER, {
                   } @radioSteps, @radioStepsLimits, NO_INVERT, YES_ROLL_OVER)

  repeat until not lockset(debugLock)
  repeat until not lockset(i2cLockId)    

  if radioVolume <> previousVolume
    Radio.VolumeSet(radioVolume)
    Lcd.Char(RADIO_VOLUME_LOC)
    
    
    BothDec(radioVolume, 2, ASCII_SPACE)
    lockclr(i2cLockId)
    lockclr(debugLock)
    return
  elseif radioSteps < previousSteps
    Radio.SeekDown
    TempString(string("SeekDown"), SEEK_TEMP_TIME, DEFAULT_TEMP_LOC)

  elseif radioSteps > previousSteps  
    Radio.SeekUp

    Pst.Str(string(ASCII_CR, "SeekUp"))        
  'radioChannel := Radio.CheckChannel
  Lcd.Char(RADIO_CHANNEL_LOC)
  result := Radio.CheckFreq
  lockclr(i2cLockId)
  'Pst.Str(string(ASCII_CR, "frequency = "))
  'Pst.Dec(result)
  BothDec(result, 6, ASCII_SPACE)
  lockclr(debugLock)

PUB AdjustDisplayType | previousDisplayType, previousSize

  repeat until not lockset(debugLock)
  Pst.Str(string(ASCII_CR, "AdjustDisplayType Method"))
  lockclr(debugLock)
  
  previousDisplayType := displayType
    
  ' displayType enumeration
'  #0, PATTERN_DISPLAY, LARSEN_DISPLAY, COLOR_DROP_DISPLAY, OFF_DISPLAY
  
  AdjustParameters(@displayType, @displayTypeLimits, NO_INVERT, YES_ROLL_OVER, {
                   } @displayType, @displayTypeLimits, NO_INVERT, YES_ROLL_OVER)
                                                              
  if displayType <> previousDisplayType
    Lcd.Char(STRIP_DISPLAY_TYPE_LOC)
    DisplayDisplayType(displayType)
    if displayType == OFF_DISPLAY
      Beep.BeeDeep
      
PUB AdjustStripBrightnessSpeed | previousBrightness, previousSpeed

  {repeat until not lockset(debugLock)
  Pst.Str(string(ASCII_CR, "AdjustStripBrightnessSpeed Method"))
  Pst.Str(string(ASCII_CR, "stripBrightness = "))
  Pst.Dec(stripBrightness)
  Pst.Str(string(ASCII_CR, "stripSpeed = "))
  Pst.Dec(stripSpeed)
  lockclr(debugLock)   }
  
  previousBrightness := stripBrightness
  previousSpeed := stripSpeed
    
  AdjustParameters(@stripBrightness, @stripBrightnessLimits, NO_INVERT, NO_ROLL_OVER, {
                   } @stripSpeed, @stripSpeedLimits, NO_INVERT, NO_ROLL_OVER)
 
  repeat until not lockset(debugLock)
  Pst.Str(string(ASCII_CR, "stripBrightness = "))
  Pst.Dec(stripBrightness)
  Pst.Str(string(ASCII_CR, "stripSpeed = "))
  Pst.Dec(stripSpeed)
                                           
  if stripBrightness <> previousBrightness
    Lcd.Char(STRIP_BRIGHTNESS_LOC)

    BothDec(stripBrightness, 3, ASCII_SPACE)            
  elseif stripSpeed <> previousSpeed    
    Lcd.Char(STRIP_SPEED_LOC)
    'BothDec(stripSpeed, 4, ASCII_SPACE)

    Pst.Dec(stripSpeed)
    LcdDecClean(stripSpeed, 4)
    if stripSpeed == 0
      Beep.BeeDeep
      'BothStr(string("Off  "))
    elseif (stripSpeed > 0 and previousSpeed < 0) or (stripSpeed < 0 and previousSpeed > 0)
      stripSpeed := 0
      Beep.BeeDeep
    elseif stripSpeed == stripSpeedLimits[MAX_OFFSET] or {
         } stripSpeed == stripSpeedLimits[MIN_OFFSET]
      stripDelay := 0
      stripRotationInterval := 0
      stripRotationTimer := cnt
      Pst.Str(string(ASCII_CR, "stripDelay = "))
      Pst.Dec(stripDelay)
    else
      result := (||stripSpeed) - 1
      stripDelay := speedToDelay[result]
      stripRotationInterval := MS_001 * stripDelay
      stripRotationTimer := cnt

      Pst.Str(string(ASCII_CR, "stripDelay = "))
      Pst.Dec(stripDelay)
  lockclr(debugLock) 
      
PUB AdjustLarsenBrightnessSpeed | previousBrightness, previousSpeed

  {repeat until not lockset(debugLock)
  Pst.Str(string(ASCII_CR, "AdjustStripBrightnessSpeed Method"))
  Pst.Str(string(ASCII_CR, "larsenBrightness = "))
  Pst.Dec(larsenBrightness)
  Pst.Str(string(ASCII_CR, "larsenSpeed = "))
  Pst.Dec(larsenSpeed)
  lockclr(debugLock)   }
  
  previousBrightness := larsenBrightness
  previousSpeed := larsenSpeed
    
  AdjustParameters(@larsenBrightness, @larsenBrightnessLimits, NO_INVERT, NO_ROLL_OVER, {
                   } @larsenSpeed, @larsenSpeedLimits, NO_INVERT, NO_ROLL_OVER)
 
  repeat until not lockset(debugLock)
  Pst.Str(string(ASCII_CR, "larsenBrightness = "))
  Pst.Dec(larsenBrightness)
  Pst.Str(string(ASCII_CR, "larsenSpeed = "))
  Pst.Dec(larsenSpeed)
                                           
  if larsenBrightness <> previousBrightness
    Lcd.Char(LARSEN_BRIGHTNESS_LOC)

    BothDec(larsenBrightness, 3, ASCII_SPACE)            
  elseif larsenSpeed <> previousSpeed    
    Lcd.Char(LARSEN_SPEED_LOC)
    'BothDec(larsenSpeed, 4, ASCII_SPACE)

    Pst.Dec(larsenSpeed)
    LcdDecClean(larsenSpeed, 4)
    if larsenSpeed == 0
      Beep.BeeDeep
      'BothStr(string("Off  "))
    elseif (larsenSpeed > 0 and previousSpeed < 0) or (larsenSpeed < 0 and previousSpeed > 0)
      larsenSpeed := 0
      Beep.BeeDeep
    elseif larsenSpeed == larsenSpeedLimits[MAX_OFFSET] or {
         } larsenSpeed == larsenSpeedLimits[MIN_OFFSET]
      larsenDelay := 0
      larsenInterval := 0
      larsenTimer := cnt
      Pst.Str(string(ASCII_CR, "larsenDelay = "))
      Pst.Dec(larsenDelay)
    else
      result := (||larsenSpeed) - 1
      larsenDelay := speedToDelay[result]
      larsenInterval := MS_001 * larsenDelay
      larsenTimer := cnt

      Pst.Str(string(ASCII_CR, "larsenDelay = "))
      Pst.Dec(larsenDelay)
  lockclr(debugLock) 
      
PUB AdjustStripColorSize | previousStripColor, previousSize

  repeat until not lockset(debugLock)
  Pst.Str(string(ASCII_CR, "AdjustStripColorSize Method"))
  lockclr(debugLock)
    
  previousStripColor := stripColor
  previousSize := stripSize
  
  
  AdjustParameters(@stripColor, @stripColorLimits, NO_INVERT, YES_ROLL_OVER, {
                 } @stripSize, @stripSizeLimits, NO_INVERT, NO_ROLL_OVER)

                                           
  if stripColor <> previousStripColor
    Lcd.Char(STRIP_COLOR_LOC)
    DisplayColorSetting(stripColor)          
  elseif stripSize <> previousSize
    Lcd.Char(STRIP_SIZE_LOC)
    repeat until not lockset(debugLock)
    BothDec(stripSize, 3, ASCII_SPACE)
    lockclr(debugLock)
  
PUB AdjustStripLarsenColors | previousForeground, previousBackground

  repeat until not lockset(debugLock)
  Pst.Str(string(ASCII_CR, "AdjustStripLarsenColors Method ******"))
  lockclr(debugLock)
    
  previousForeground := larsenForeIndex
  previousBackground := larsenBackIndex
  
  LarsenColorAdjust
                                           
  if larsenForeIndex <> previousForeground
    if larsenForeIndex == larsenBackIndex
      'Beep.BeepBeep
      LarsenColorAdjust ' get next color so they aren't equal
    Lcd.Char(FOREGROUND_COLOR_LOC)
    DisplayColorSetting(larsenForeIndex)         
  elseif larsenBackIndex <> previousBackground
    if larsenBackIndex == larsenForeIndex
      'Beep.BeepBeep
      LarsenColorAdjust ' get next color so they aren't equal
    Lcd.Char(BACKGROUND_COLOR_LOC)   
    DisplayColorSetting(larsenBackIndex)     

PRI LarsenColorAdjust

  AdjustParameters(@larsenForeIndex, @larsenForeIndexLimits, NO_INVERT, YES_ROLL_OVER, {
                 } @larsenBackIndex, @larsenBackIndexLimits, NO_INVERT, YES_ROLL_OVER)
  
PUB AdjustStripLarsenSizes | previousSize, previousInUse

  repeat until not lockset(debugLock)
  Pst.Str(string(ASCII_CR, "AdjustStripLarsenSizes Method"))
  lockclr(debugLock)
    
  previousSize := larsenSize
  previousInUse := larsensInUse
            
  AdjustParameters(@larsenSize, @larsenSizeLimits, NO_INVERT, NO_ROLL_OVER, {
                 } @larsensInUse, @larsensInUseLimits, NO_INVERT, NO_ROLL_OVER)
                                          
  if larsenSize <> previousSize  
    Lcd.Char(LARSEN_SIZE_LOC)
    repeat until not lockset(debugLock)
    BothDec(larsenSize, 3, ASCII_SPACE)
    lockclr(debugLock)
           
  elseif larsensInUse <> previousInUse
    Lcd.Char(LARSEN_PATTERNS_LOC)   
    repeat until not lockset(debugLock)
    BothDec(larsensInUse, 3, ASCII_SPACE)
    lockclr(debugLock)
  
           
{PUB AdjustXmasBrightnessSpeed

  Pst.Str(string(ASCII_CR, "AdjustXmasBrightnessSpeed Method"))
  Pst.Str(string(ASCII_CR, "stripBrightness = "))
  Pst.Dec(stripBrightness)
  Pst.Str(string(ASCII_CR, "stripSpeed = "))
  Pst.Dec(stripSpeed)
  if button == Keypad#CENTER_BUTTON
    return
       
PUB AdjustXmasColorSize

  Pst.Str(string(ASCII_CR, "AdjustStripBrightnessSpeed Method"))
  Pst.Str(string(ASCII_CR, "stripBrightness = "))
  Pst.Dec(stripBrightness)
  Pst.Str(string(ASCII_CR, "stripSpeed = "))
  Pst.Dec(stripSpeed)
  if button == Keypad#CENTER_BUTTON
    return
    
PUB AdjustXmasText

  Pst.Str(string(ASCII_CR, "AdjustStripBrightnessSpeed Method"))
  Pst.Str(string(ASCII_CR, "stripBrightness = "))
  Pst.Dec(stripBrightness)
  Pst.Str(string(ASCII_CR, "stripSpeed = "))
  Pst.Dec(stripSpeed)
  if button == Keypad#CENTER_BUTTON
    return
}    
PUB AdjustBacklight

  'Pst.Str(string(ASCII_CR, "AdjustBacklight Method"))
  'Pst.Str(string(ASCII_CR, "LcdMode = "))
  'Pst.Dec(lcdMode)
  result := lcdMode
  if button == Keypad#CENTER_BUTTON
    return

  if button == Keypad#LEFT_BUTTON or button == Keypad#RIGHT_BUTTON
    if LcdMode
      LcdMode := OFF
      Lcd.Char(LCD_BACKLIGHT_OFF) 
    else
      LcdMode := ON
      Lcd.Char(LCD_BACKLIGHT_ON) 
  elseif button == Keypad#UP_BUTTON
    LcdMode := ON
    Lcd.Char(LCD_BACKLIGHT_ON)
  elseif button == Keypad#DOWN_BUTTON
    LcdMode := OFF
    Lcd.Char(LCD_BACKLIGHT_OFF)
  'Pst.Str(string(ASCII_CR, "LcdMode = "))
  'Pst.Dec(lcdMode)
  if lcdMode <> result
    repeat until not lockset(debugLock)
    Pst.Str(string(ASCII_CR, "LcdMode = "))
    Pst.Dec(lcdMode)
    Lcd.Char(BACKLIGHT_LOC)
    case lcdMode
      ON:
        Lcd.Str(@on_text)
        Beep.TurnOn
      OFF:
        Lcd.Str(@off_text)
        Beep.TurnOff
    lockclr(debugLock)
              
PUB AdjustYearMonth

  changableTimeFlag := YEAR_MONTH_ADJUST
  AdjustParameters(@editedTime + (4 * YEAR_INDEX), @yearLimits, NO_INVERT, NO_ROLL_OVER, {
                   } @editedTime + (4 * MONTH_INDEX), @monthLimits, NO_INVERT, YES_ROLL_OVER)
  repeat until not lockset(debugLock)
  if editedTime[YEAR_INDEX] <> lastReadTime[YEAR_INDEX]
    timeChangedFlag[YEAR_INDEX] := 1
    Lcd.Char(ADJUST_YEAR_LOC)
    'LcdPad(year, 2, "0")
    Pst.Str(string(ASCII_CR, "editedTime[YEAR_INDEX] = "))
    Pst.Dec(editedTime[YEAR_INDEX])
    LcdPad(editedTime[YEAR_INDEX], 4, "0")         
  if editedTime[MONTH_INDEX] <> lastReadTime[MONTH_INDEX]
    timeChangedFlag[MONTH_INDEX] := 1                 
    Lcd.Char(ADJUST_MONTH_LOC)
    'LcdPad(month, 2, ASCII_SPACE)
    Pst.Str(string(ASCII_CR, "editedTime[MONTH_INDEX] = "))
    Pst.Dec(editedTime[MONTH_INDEX])
    LcdPad(editedTime[MONTH_INDEX], 2, "0") 
  lockclr(debugLock)                 
   
PUB AdjustDateDay

  changableTimeFlag := DATE_DAY_ADJUST
  'longMove(@editedTime, @lastReadTime, TIME_ARRAY_SIZE)
  AdjustParameters(@editedTime + (4 * DATE_INDEX), @dateLimits, NO_INVERT, YES_ROLL_OVER, {
                   } @editedTime + (4 * DAY_INDEX), @dayLimits, NO_INVERT, YES_ROLL_OVER)
  if editedTime[DATE_INDEX] <> lastReadTime[DATE_INDEX] 'TIME_ARRAY_SIZE]
    timeChangedFlag[DATE_INDEX] := 1
    Lcd.Char(ADJUST_DATE_LOC)
    LcdPad(editedTime[DATE_INDEX], 2, ASCII_SPACE)                                      
  if editedTime[DAY_INDEX] <> lastReadTime[DAY_INDEX]
    timeChangedFlag[DAY_INDEX] := 1
    Lcd.Char(ADJUST_DAY_LOC)
    Lcd.Dec(editedTime[DAY_INDEX])
    'LcdPad(editedTime[DAY_INDEX], 1, ASCII_SPACE)                 
    
PUB AdjustHourMode

  changableTimeFlag := HOUR_MODE_ADJUST
  result := mode
  'result := hourMode 
  'longMove(@editedTime, @lastReadTime, TIME_ARRAY_SIZE)
  AdjustParameters(@editedTime + (4 * HOUR_INDEX), @hourLimits, NO_INVERT, YES_ROLL_OVER, {
                   } @hourMode, @hourModeLimits, NO_INVERT, NO_ROLL_OVER)
  if editedTime[HOUR_INDEX] <> lastReadTime[HOUR_INDEX] 
    timeChangedFlag[HOUR_INDEX] := 1                
    Lcd.Char(ADJUST_HOUR_LOC)
    LcdPad(editedTime[HOUR_INDEX], 2, ASCII_SPACE)
  if hourMode <> result             
    Lcd.Char(ADJUST_TIME_MODE_LOC)
    'LcdPad(mode, 1, ASCII_SPACE)
    repeat until not lockset(debugLock)
    case hourMode       
      AM_PM_TIME:
        BothStr(@am_pm)
      MILITARY_TIME:
        BothStr(@military)
    lockclr(debugLock)
       
PUB AdjustMinuteSecond

  changableTimeFlag := MINUTE_SECOND_ADJUST
  'longMove(@editedTime, @lastReadTime, TIME_ARRAY_SIZE)
  AdjustParameters(@editedTime + (4 * MINUTE_INDEX), @timeLimits, NO_INVERT, YES_ROLL_OVER, {
                   } @editedTime + (4 * SECOND_INDEX), @timeLimits, NO_INVERT, YES_ROLL_OVER)
  repeat until not lockset(debugLock)
  if editedTime[MINUTE_INDEX] <> lastReadTime[MINUTE_INDEX] 'TIME_ARRAY_SIZE]
    timeChangedFlag[MINUTE_INDEX] := 1
    Lcd.Char(ADJUST_MINUTE_LOC)
    LcdPad(editedTime[MINUTE_INDEX], 2, "0")                
  if editedTime[SECOND_INDEX] <> lastReadTime[SECOND_INDEX]
    timeChangedFlag[SECOND_INDEX] := 1
    Lcd.Char(ADJUST_SECOND_LOC)
    LcdPad(editedTime[SECOND_INDEX], 2, "0")
  lockclr(debugLock)                 
  
PUB AdjustOther

  repeat until not lockset(debugLock)
  Pst.Str(string(ASCII_CR, "AdjustOther Method"))
  lockclr(debugLock)
    
{PUB DisplayClock(lcdLocation)

  Lcd.Char(lcdLocation)
  LcdPad(12, 2, ASCII_SPACE)
  Lcd.Char(":")
  LcdPad(59, 2, "0")
  Lcd.Char(":")
  LcdPad(59, 2, "0")
  Lcd.Char("A")
  Lcd.Char("M")
 } 
PUB LoadChase | localChannel, temp

    localChannel := 0
    repeat
      repeat result from 0 to 7
        repeat temp from $1F to $7F step $1F
          stripOutputBuffer[localChannel++] := color1[result] * temp          
        repeat temp from $60 to $1F step $1F
          stripOutputBuffer[localChannel++] := color1[result] * temp
    while localChannel < CHANNELS
    
{PUB BuildLongColors | longIndex, byteIndex, byteColorPtr

  byteColorPtr := @blackSetting

  repeat longIndex from 0 to 8
    blackSettingL[longIndex] := 0
    repeat 3
      blackSettingL[longIndex] <<= 8
      blackSettingL[longIndex] += byte[byteColorPtr]
}      
PRI MonitorLeds 
   
  pDisplayType := -1
  pStripSize := -1
  pStripBrightness := -1
  pStripColor := -1
  pLarsenForeIndex := -1
  pLarsenBackIndex := -1
  pLarsenSize := -1
  pLarsensInUse := -1
  repeat
    ' Freeze parameters to compare so they don't change between
    ' the comparison and when the previous values are saved.
    freezeStripSize := stripSize
    freezeStripBrightness := stripBrightness
    freezeDisplayType := displayType
    freezeStripColor := stripColor
    freezeLarsenForeIndex := larsenForeIndex
    freezeLarsenBackIndex := larsenBackIndex
    freezeLarsenSize := larsenSize
    freezeLarsenInUse := larsensInUse
    if freezeDisplayType <> pDisplayType or freezeStripSize <> pStripSize or {
     } freezeStripBrightness <> pStripBrightness or freezeStripColor <> pStripColor or {
     } freezeLarsenForeIndex <> pLarsenForeIndex or freezeLarsenBackIndex <> {
     } pLarsenBackIndex or freezeLarsenSize <> pLarsenSize or freezeLarsenInUse <> {
     } pLarsensInUse
  
      
      'blendFlag := true 'false
      pDisplayType := freezeDisplayType
      pStripSize := freezeStripSize
      pStripBrightness := freezeStripBrightness
      pStripColor := freezeStripColor
      pLarsenForeIndex := freezeLarsenForeIndex
      pLarsenBackIndex := freezeLarsenBackIndex
      pLarsenSize := freezeLarsenSize
      pLarsensInUse := freezeLarsenInUse
      repeat until not lockset(debugLock)
      Pst.Str(string(ASCII_CR, " ****** New Parameter ******"))
      Pst.Str(string(ASCII_CR, "fStripSize = "))
      Pst.Dec(freezeStripSize)
      Pst.Str(string(ASCII_CR, "fStripBrightness = "))
      Pst.Dec(freezeStripBrightness)
      Pst.Str(string(ASCII_CR, "fDisplayType = "))
      Pst.Dec(freezeDisplayType)
      Pst.Str(string(ASCII_CR, "fStripStripColor = "))
      Pst.Dec(freezeStripColor)
      
      lockclr(debugLock)
      case freezeDisplayType
        PATTERN_DISPLAY:
          ledPointer := @patternSingle
          UpdatePattern
        LARSEN_DISPLAY:
          LoadLarsen
        COLOR_DROP_DISPLAY:
        OFF_DISPLAY:
          ledPointer := @patternSingle
          longfill(@patternSingle, 0, 4)
    case freezeDisplayType
      PATTERN_DISPLAY:
        CheckStripRotation(@stripOutputBuffer)
      LARSEN_DISPLAY:
        MoveLarsen
      COLOR_DROP_DISPLAY:
      OFF_DISPLAY:
        'ledPointer := @patternSingle
        'longfill(@patternSingle, 0, 4)
     

PRI UpdatePattern
   
  case freezeStripColor          
    {OFF_STRIP:
      'result := @patternSingle 
      patternSingle[0] := blackSettingL
      patternSingle[1] := blackSettingL
      patternSingle[2] := blackSettingL
      patternSingle[3] := blackSettingL }
    RED_STRIP..INDIGO_STRIP:
      repeat until not lockset(debugLock)
      Pst.Str(string(ASCII_CR, "Loading patternSingleBuffer"))
      lockclr(debugLock)
      
      if stripSize
        
        patternSingle[0] := blackSettingL[freezeStripColor]
        patternSingle[1] := DimLong(blackSettingL[freezeStripColor], DEFAULT_VERY_DIM)
        patternSingle[2] := blackSettingL[freezeStripColor]
        patternSingle[3] := DimLong(blackSettingL[freezeStripColor], DEFAULT_VERY_DIM)      
      else
       
        patternSingle[0] := blackSettingL[freezeStripColor]
        patternSingle[1] := blackSettingL[freezeStripColor]
        patternSingle[2] := blackSettingL[freezeStripColor]
        patternSingle[3] := blackSettingL[freezeStripColor]
   
    RAINBOW_STRIP:
      ledPointer := @rainbowPattern
      repeat until not lockset(debugLock)
      Pst.Str(string(ASCII_CR, "Loading patternSingleBuffer"))
      lockclr(debugLock)
      
  ExpandPattern(ledPointer, freezeStripSize, true)
   
PUB CheckLarsenMovement(localPtr)

  'repeat until not lockset(debugLock)
  'lockclr(debugLock)
  if larsenSpeed == 0
    larsenTimer := cnt
    return
  elseif cnt - larsenTimer < larsenInterval
    return
  else  
    larsenTimer := cnt '+= larsenInterval
    MoveLarsen
    
PRI MoveLarsen | localIndex, fadeIndex

  repeat until not lockset(debugLock)
  Pst.Str(string(ASCII_CR, "MoveLarsen Method ******"))
  
  longfill(@expandedPattern, activeBack, patternSize)

  repeat result from 0 to larsenSize
    Pst.Str(string(ASCII_CR, "larsenFade["))
    Pst.Dec(result)
    Pst.Str(string("] = "))
    Hex(larsenFade[result])
  
  if larsenDirection == 1 and larsenIndex == patternSize - 1
    larsenDirection := -1
    Pst.Str(string(ASCII_CR, "reverse direction"))
  elseif larsenDirection == -1 and larsenIndex == 0
    larsenDirection := 1
  larsenIndex += larsenDirection
  expandedPattern[larsenIndex] := larsenFade[0] 
  Pst.Str(string(ASCII_CR, "expandedPattern["))
  Pst.Dec(larsenIndex)
  Pst.Str(string("] = "))
  Hex(expandedPattern[larsenIndex])
  localIndex := larsenIndex - 1
  fadeIndex := 1
  repeat while localIndex => 0 and fadeIndex < larsenSize
    expandedPattern[localIndex--] := larsenFade[fadeIndex++]
    Pst.Str(string(ASCII_CR, "expandedPattern["))
    Pst.Dec(localIndex + 1)
    Pst.Str(string("] = "))
    Hex(expandedPattern[localIndex + 1])
  localIndex := larsenIndex + 1
  fadeIndex := 1
  repeat while localIndex < patternSize and fadeIndex < larsenSize
    expandedPattern[localIndex++] := larsenFade[fadeIndex++]
    Pst.Str(string(ASCII_CR, "expandedPattern["))
    Pst.Dec(localIndex - 1)
    Pst.Str(string("] = "))
    Hex(expandedPattern[localIndex - 1])
    
  lockclr(debugLock) 
  LoadPatternToDefaultBuffer(@expandedPattern)
  
{PRI LoadLarsen | pseudoQuantum[3], pForegroundRed, pForegroundGreen, {
               } pForegroundBlue, pBackgroundRed, pBackgroundGreen, pBackgroundBlue, {
               } localMax, localColor[3], localIndex  
  repeat until not lockset(debugLock)
  Pst.Str(string(ASCII_CR, "LoadLarsen Method"))
       
  patternSize := CHANNELS / larsensInUse
  expandedPattern[patternSize] := TERMINATOR
  larsenSize #>= patternSize
  
  Pst.Str(string(ASCII_CR, "patternSize = "))
  Pst.Dec(patternSize)
  activeFore := blackSettingL[larsenForeIndex]
  activeBack := blackSettingL[larsenBackIndex]
  BustColors(activeFore, @pForegroundRed)
  BustColors(activeBack, @pBackgroundRed)
  Pst.Str(string(ASCII_CR, "larsenForeIndex = "))
  Hex(larsenForeIndex)
  Pst.Str(string(ASCII_CR, "pForegroundRGB = "))
  Hex(pForegroundRed)
  Pst.Str(string(", "))
  Hex(pForegroundGreen)
  Pst.Str(string(", "))
  Hex(pForegroundBlue)
  
  Pst.Str(string(ASCII_CR, "larsenBackIndex = "))
  Hex(larsenBackIndex)
  Pst.Str(string(ASCII_CR, "pBackgroundRGB = "))
  Hex(pBackgroundRed)
  Pst.Str(string(", "))
  Hex(pBackgroundGreen)
  Pst.Str(string(", "))
  Hex(pBackgroundBlue)
  
  longmove(@localColor, @pBackgroundRed, 3)
  localMax := 255
  repeat localIndex from 0 to 2
    if pForegroundRed[localIndex] + pBackgroundRed[localIndex] > localMax
      localMax := pForegroundRed[localIndex] + pBackgroundRed[localIndex]
  Pst.Str(string(ASCII_CR, "localMax = "))
  Hex(localMax)
  Pst.Str(string(", = "))
  Pst.Dec(localMax)
          
  if localMax > 255
    repeat localIndex from 0 to 2
      localColor[localIndex] <<= 8
      localColor[localIndex] /= localMax
      pForegroundRed[localIndex] <<= 8
      pBackgroundRed[localIndex] <<= 8
      pForegroundRed[localIndex] /= localMax
      pBackgroundRed[localIndex] /= localMax

  '** do I really want foreground and background added?
  repeat localIndex from 0 to 2    
    pseudoQuantum[localIndex] := ((pForegroundRed[localIndex] + pBackgroundRed[localIndex]) {
                               } * 1000) / larsenSize
    Pst.Str(string(ASCII_CR, "pseudoQuantum["))
    Pst.Dec(localIndex)
    Pst.Str(string("] = "))
    Hex(pseudoQuantum[localIndex])
    Pst.Str(string(", "))
    Pst.Dec(pseudoQuantum[localIndex])

                            
  activeBack := Leds.make_rgb(localColor[0], localColor[1], localColor[2])
  Pst.Str(string(ASCII_CR, "activeBack = "))
  Hex(activeBack)
  
  repeat result from 0 to larsenSize - 1
    repeat localIndex from 0 to 2
      localColor[localIndex] := (pseudoQuantum[localIndex] * (larsenSize - result)) / 1000
    larsenFade[result] := Leds.make_rgb(localColor[0], localColor[1], localColor[2]) 
    Pst.Str(string(ASCII_CR, "larsenFade["))
    Pst.Dec(result)
    Pst.Str(string("] = "))
    Hex(larsenFade[result])
    Pst.Str(string(", "))
    Pst.Dec(larsenFade[result])
  '*** think about fade colors only foreground component should fade
  larsenDirection := 1
  larsenIndex := 0
  lockclr(debugLock)
}
PRI LoadLarsen

  repeat until not lockset(debugLock)
  Pst.Str(string(ASCII_CR, "LoadLarsen Method ******"))
       
  patternSize := CHANNELS / larsensInUse
  expandedPattern[patternSize] := TERMINATOR
  larsenSize <#= patternSize
  
  Pst.Str(string(ASCII_CR, "patternSize = "))
  Pst.Dec(patternSize)
  lockclr(debugLock)

  activeFore := blackSettingL[larsenForeIndex]
  activeBack := blackSettingL[larsenBackIndex]
  
  'MergeColors(activeFore, activeBack, @larsenFade, larsenSize)
  FadeColors(activeFore, activeBack, @larsenFade, larsenSize)
  
  larsenDirection := 1
  larsenIndex := 0
  
PUB MergeColors(initialColor, finalColor, distinationPtr, localSize) | tempColor[3], {
    } colorMultiplier, localIndex, initialColors[3], finalColors[3], pseudoIncrements[3]
'' localSize should include the initialColor but not finalColor.
  repeat until not lockset(debugLock)
  Pst.Str(string(ASCII_CR, "MergeColors Method ******"))
  Pst.Str(string(ASCII_CR, "initialColor = "))
  Hex(initialColor)
  Pst.Str(string(ASCII_CR, "finalColor = "))
  Hex(finalColor)
    
  BustColors(initialColor, @initialColors)
  BustColors(finalColor, @finalColors)
   
  ' find incremental color change
  ' use the "1000" mulitplier to reduce rounding errors
  ' Note we divide by multiplier prior to assigning color
  ' to expanded pattern buffer.
  repeat result from 0 to 2
    pseudoIncrements[result] := ((finalColors[result] - initialColors[result])* 1000) / localSize 
    Pst.Str(string(ASCII_CR, "pseudoIncrements["))
    Pst.Dec(result)
    Pst.Str(string("] = "))
    Pst.Dec(pseudoIncrements[result])
  
  colorMultiplier := 1
   
  ' assign initial color
  long[distinationPtr] := initialColor
   
  ' assign between colors
  ' final color not copied here. It will be copied as the
  ' initial color of the next loop
  repeat localSize - 1
    distinationPtr += 4
    repeat result from 0 to 2
      tempColor[result] := initialColors[result] + (pseudoIncrements[result] * {
                         } colorMultiplier) / 1000
    ' assign altered color
    long[distinationPtr] := Leds.make_rgb(tempColor[0], tempColor[1], tempColor[2])
    Pst.Str(string(ASCII_CR, "long["))
    Pst.Dec(distinationPtr)
    Pst.Str(string("] = "))
    Hex(long[distinationPtr])
    colorMultiplier++
  lockclr(debugLock)
  
PUB FadeColors(initialColor, finalColor, distinationPtr, localSize) | tempColor[3], {
    } colorMultiplier, localIndex, initialColors[3], finalColors[3], pseudoIncrements[3]
'' localSize should include the initialColor but not finalColor.
  repeat until not lockset(debugLock)
  Pst.Str(string(ASCII_CR, "FadeColors Method ******"))
  Pst.Str(string(ASCII_CR, "initialColor = "))
  Hex(initialColor)
  Pst.Str(string(ASCII_CR, "finalColor = "))
  Hex(finalColor)

  lockclr(debugLock)
  
  repeat localSize - 1
    long[distinationPtr] := initialColor
    MergeColors(initialColor, finalColor, @tempColor, 2)
    initialColor := tempColor[1]
    repeat until not lockset(debugLock)
    Pst.Str(string(ASCII_CR, "FadeColors long["))
    Pst.Dec(distinationPtr)
    Pst.Str(string("] = "))
    Hex(long[distinationPtr])
    lockclr(debugLock) 
    distinationPtr += 4 
  
PUB BustColors(localColor, localPtr)

  long[localPtr] := (localColor >> 16) & $FF
  long[localPtr + 4] := (localColor >> 8) & $FF
  long[localPtr + 8] := localColor & $FF

{PRI PseudoDimLong(localColor, localBrightness, localPtr, localPseudo) | localRed, localGreen, localBlue
 
  if localBrightness == MAX_STRIP_BRIGHTNESS
    return localColor

  localRed := (((localColor >> 16) & $FF) * localBrightness) >> 8
  localGreen := (((localColor >> 8) & $FF) * localBrightness) >> 8
  localBlue := ((localColor & $FF) * localBrightness) >> 8
  result := Leds.make_rgb(localRed, localGreen, localBlue)
 }   
PUB FindSize(localPtr)

  repeat while long[localPtr][result] <> TERMINATOR and result <> LARGEST_POSSIBLE_PATTERN
    result++
    
{PUB LoadSpoke(spokeId, color) : localIndex

  'spokeId //= SPOKES
  localIndex := spokeId * LEDS_PER_SPOKE
  repeat LEDS_PER_SPOKE  
    stripOutputBuffer[localIndex++] := color
    'leds.set(localChannel++, color)
    if localIndex > LAST_CHANNEL
      localIndex := 0
  SendToStripOutput(@stripOutputBuffer)
}
PUB LoadPatternToDefaultBuffer(sourcePtr) | localIndex', localColor

  LoadPattern(@rawColorsBuffer, sourcePtr)
  'LoadPattern(@stripOutputBuffer, sourcePtr)
  
PUB LoadPattern(distinationPtr, sourcePtr) | localIndex', localColor

  patternSize := FindSize(sourcePtr)
  if patternSize > LARGEST_POSSIBLE_PATTERN
    return

  if patternSize > CHANNELS
    patternsInBuffer := 1
  else    
    patternsInBuffer := CHANNELS / patternSize
    if CHANNELS // patternSize
      patternsInBuffer++

  activeBufferSize := patternsInBuffer * patternSize
   
  localIndex := 0

  repeat patternsInBuffer  
    longmove(distinationPtr + (4 * localIndex++ * patternSize), sourcePtr, patternSize)

  repeat until not lockset(debugLock)
  Pst.Str(string(ASCII_CR, "LoadPattern Method ******"))
  Pst.Str(string(ASCII_CR, "stripSizeLimits[MAX_OFFSET] = "))
  Pst.Dec(stripSizeLimits[MAX_OFFSET])
  Pst.Str(string(ASCII_CR, "patternSize = "))
  Pst.Dec(patternSize)
  Pst.Str(string(ASCII_CR, "patternsInBuffer = "))
  Pst.Dec(patternsInBuffer)
  Pst.Str(string(ASCII_CR, "activeBufferSize = "))
  Pst.Dec(activeBufferSize)
  Pst.Str(string(ASCII_CR, "stripSize = "))
  Pst.Dec(stripSize)
  Pst.Str(string(ASCII_CR, "distinationPtr = "))
  Pst.Dec(distinationPtr)
  Pst.Str(string(ASCII_CR, "sourcePtr = "))
  Pst.Dec(sourcePtr)
  Pst.Str(string(ASCII_CR, "localIndex - 1 = "))
  Pst.Dec(localIndex - 1)
  lockclr(debugLock)
  
  SendToStripOutput(distinationPtr)
      
PUB ExpandPattern(localPtr, fillLeds, blendEndsFlag) | localIndex, localTemp, continueFlag, {
                 } tempColor[3], localRed[3], localGreen[3], localBlue[3], distinationPtr, {
                 } colorMultiplier, finalIndex

  patternSize := FindSize(localPtr)
  stripSizeLimits[MAX_OFFSET] := (CHANNELS / patternSize) - 1
  fillLeds <#= stripSizeLimits[MAX_OFFSET]
  localTemp := fillLeds + 1
  continueFlag := 1
  distinationPtr := @expandedPattern
  repeat until not lockset(debugLock)
  Pst.Str(string(ASCII_CR, "ExpandPattern Method ******"))
  Pst.Str(string(ASCII_CR, "stripSizeLimits[MAX_OFFSET] = "))
  Pst.Dec(stripSizeLimits[MAX_OFFSET])
  Pst.Str(string(ASCII_CR, "patternSize = "))
  Pst.Dec(patternSize)
  Pst.Str(string(ASCII_CR, "patternsInBuffer = "))
  Pst.Dec(patternsInBuffer)
  Pst.Str(string(ASCII_CR, "activeBufferSize = "))
  Pst.Dec(activeBufferSize)
  Pst.Str(string(ASCII_CR, "fillLeds = "))
  Pst.Dec(fillLeds)
  Pst.Str(string(ASCII_CR, "localPtr = "))
  Pst.Dec(localPtr)
  Pst.Str(string(ASCII_CR, "@rainbowPattern = "))
  Pst.Dec(@rainbowPattern)
  Pst.Str(string(ASCII_CR, "@patternSingle = "))
  Pst.Dec(@patternSingle)
  Pst.Str(string(ASCII_CR, "@rawColorsBuffer = "))
  Pst.Dec(@rawColorsBuffer)
  Pst.Str(string(ASCII_CR, "@stripOutputBuffer = "))
  Pst.Dec(@stripOutputBuffer)
  lockclr(debugLock)
    
  repeat
    result := localTemp * patternSize
    if result > LARGEST_POSSIBLE_PATTERN
      localTemp--
    else
      continueFlag := 0
  while continueFlag

  if localTemp =< 1
    LoadPatternToDefaultBuffer(localPtr)
    return -1

  if blendEndsFlag 
    finalIndex := patternSize - 1
  else
    finalIndex := patternSize - 2

  repeat until not lockset(debugLock)
  repeat localIndex from 0 to finalIndex
    tempColor[0] := long[localPtr][localIndex]
    Pst.Str(string(ASCII_CR, "tempColor[0] = "))
    Hex(tempColor[0])
    if localIndex == patternSize - 1
      tempColor[1] := long[localPtr]
    else  
      tempColor[1] := long[localPtr][localIndex + 1]

    ' find initial color  
    localRed[0] := (tempColor[0] >> 16) & $FF
    localGreen[0] := (tempColor[0] >> 8) & $FF
    localBlue[0] := tempColor[0] & $FF

    ' find final color
    localRed[1] := (tempColor[1] >> 16) & $FF
    localGreen[1] := (tempColor[1] >> 8) & $FF
    localBlue[1] := tempColor[1] & $FF

    ' find incremental color change
    ' use the "1000" mulitplier to reduce rounding errors
    ' Note we divide by multiplier prior to assigning color
    ' to expanded pattern buffer.
    localRed[2] := ((localRed[1] - localRed[0]) * 1000) /  localTemp
    localGreen[2] := ((localGreen[1] - localGreen[0]) * 1000) /  localTemp
    localBlue[2] := ((localBlue[1] - localBlue[0]) * 1000) /  localTemp
    
    colorMultiplier := 1
    
    ' assign initial color
    long[distinationPtr] := tempColor[0]

    ' assign between colors
    ' final color not copied here. It will be copied as the
    ' initial color of the next loop
    repeat localTemp - 1 
      localRed[1] := localRed[0] + (localRed[2] * colorMultiplier) / 1000  
      localGreen[1] := localGreen[0] + (localGreen[2] * colorMultiplier) / 1000
      localBlue[1] := localBlue[0] + (localBlue[2] * colorMultiplier) / 1000
      tempColor[0] := Leds.make_rgb(localRed[1], localGreen[1], localBlue[1])
      distinationPtr += 4
      ' assign altered color
      long[distinationPtr] := tempColor[0]
      colorMultiplier++
  lockclr(debugLock)  
      
  distinationPtr += 4
  
  ' mark end of pattern
  long[distinationPtr] := TERMINATOR
  LoadPatternToDefaultBuffer(@expandedPattern)
  
{PUB DisplayPattern(localIndex) | localColor, localPtr

  'spokeId //= SPOKES
  Pst.str(string(ASCII_CR, "DisplayPattern method", 13, "localIndex = "))
  Pst.dec(localIndex)
  repeat while localIndex < 0
    localIndex += patternSize
  repeat while localIndex => patternSize
    localIndex -= patternSize
    
  result := localIndex
  Pst.str(string(ASCII_CR, "localIndex = "))
  Pst.dec(localIndex)
  
  localPtr := @stripOutputBuffer
  Pst.str(string(ASCII_CR, "long at location "))
  Hex(localPtr)
  Pst.str(string(", = "))
  repeat SPOKES  
    longmove(localPtr, @patternBuffer + (4 * localIndex), LEDS_PER_SPOKE)    
    Hex(long[localPtr])
    Pst.str(string(", "))
    Hex(long[localPtr + 4])
    Pst.str(string(", "))
    Hex(long[localPtr + 8])
    Pst.str(string(", "))
    Hex(long[localPtr + 12])
    
    localPtr += 4 * LEDS_PER_SPOKE
     
  SendToStripOutput
}  
{PUB Rotate(rotationAmount) '| totalToRotate

  if rotationAmount > 0
    longmove(@spokeBuffer, @stripOutputBuffer, LEDS_PER_SPOKE)
    longmove(@stripOutputBuffer, @stripOutputBuffer + (4 * LEDS_PER_SPOKE), LEDS_PER_SPOKE * (SPOKES - 1))
    longmove(@stripOutputBuffer + (4 * (LEDS_PER_SPOKE * (SPOKES - 1))), @spokeBuffer, LEDS_PER_SPOKE)
  else
    longmove(@spokeBuffer, @stripOutputBuffer + (4 * (LEDS_PER_SPOKE * (SPOKES - 1))), LEDS_PER_SPOKE)
    longmove(@stripOutputBuffer + (4 * LEDS_PER_SPOKE), @stripOutputBuffer, LEDS_PER_SPOKE * (SPOKES - 1))
    longmove(@stripOutputBuffer, @spokeBuffer, LEDS_PER_SPOKE)
  'totalToRotate := (LEDS_PER_SPOKE * SPOKES) - 1
  SendToStripOutput(@stripOutputBuffer)
  
PUB RotateBuffer(rotationAmount) '| totalToRotate

  if rotationAmount > 0
    longmove(@rotationBuffer, @stripOutputBuffer, rotationAmount)
    longmove(@stripOutputBuffer, @stripOutputBuffer + (4 * rotationAmount), CHANNELS - rotationAmount)
    longmove(@stripOutputBuffer + (4 * (CHANNELS - rotationAmount)), @rotationBuffer, rotationAmount)
  else
    longmove(@spokeBuffer, @stripOutputBuffer + (4 * (LEDS_PER_SPOKE * (SPOKES - 1))), LEDS_PER_SPOKE)
    longmove(@stripOutputBuffer + (4 * LEDS_PER_SPOKE), @stripOutputBuffer, LEDS_PER_SPOKE * (SPOKES - 1))
    longmove(@stripOutputBuffer, @spokeBuffer, LEDS_PER_SPOKE)
  'totalToRotate := (LEDS_PER_SPOKE * SPOKES) - 1
  SendToStripOutput(@stripOutputBuffer)
}
PUB CheckStripRotation(localPtr)

  'repeat until not lockset(debugLock)
  'lockclr(debugLock)
  if stripSpeed == 0
    stripRotationTimer := cnt
    return
  elseif cnt - stripRotationTimer < stripRotationInterval
    return
  else  
    stripRotationTimer := cnt '+= stripRotationInterval
    RotateStripBuffer(localPtr)
    
PUB RotateStripBuffer(localPtr)

  {repeat until not lockset(debugLock)
  Pst.str(string(ASCII_CR, "stripSpeed = "))
  Pst.dec(stripSpeed)
  Pst.str(string(ASCII_CR, "activeBufferSize = "))
  Pst.dec(activeBufferSize)
  Pst.str(string(ASCII_CR, "localPtr = "))
  Pst.dec(localPtr)
  Pst.str(string(ASCII_CR, "rotate "))     }
  if stripSpeed > 0
    'Pst.str(string("for"))  
    result := long[localPtr][activeBufferSize - 1]
    longmove(localPtr + 4, localPtr, activeBufferSize - 1)
    long[localPtr] := result
  else
    'Pst.str(string("back"))  
    result := long[localPtr]
    longmove(localPtr, localPtr + 4, activeBufferSize - 1)
    long[localPtr][activeBufferSize - 1] := result
  'Pst.str(string("ward"))
  'lockclr(debugLock)
  RefreshStripOutput(localPtr)
    
PUB RefreshStripOutput(localPtr)
' buffer not adjusted for brightness
' any buffer may be used as output buffer
' stripOutputBuffer should be used in most cases

  repeat result from 0 to LAST_CHANNEL
    Leds.set(result, long[localPtr][result])
    'Leds.set(result, long[@stripOutputBuffer][result])
    
PUB SendToStripOutput(localPtr)

  'activeStripBuffer := distinationPtr
  AdjustStripBrightness(localPtr)
  repeat result from 0 to LAST_CHANNEL
    Leds.set(result, long[@stripOutputBuffer][result]) 'stripOutputBuffer[result])

PUB AdjustStripBrightness(localPtr) | localRed, localGreen, localBlue, tempColor
 
  if stripBrightness == MAX_STRIP_BRIGHTNESS and STRAND_TYPE <> CHINA_STAND
    return
  repeat result from 0 to activeBufferSize - 1
    tempColor := long[localPtr][result] 
    localRed := (((tempColor >> 16) & $FF) * stripBrightness) >> 8
    localGreen := (((tempColor >> 8) & $FF) * stripBrightness) >> 8
    localBlue := ((tempColor & $FF) * stripBrightness) >> 8
    if STRAND_TYPE == CHINA_STAND
      'long[@stripOutputBuffer][result] := Leds.make_rgb(localBlue, localGreen, localRed)  'gbr w/false
      'long[@stripOutputBuffer][result] := Leds.make_rgb(localGreen, localRed, localBlue) 'brg w/false
      long[@stripOutputBuffer][result] := Leds.make_rgb(localRed, localBlue, localGreen) 'rgb! w/false
      'long[@stripOutputBuffer][result] := Leds.make_rgb(localRed, localGreen, localBlue) 'gbr w/true 
    else
      long[@stripOutputBuffer][result] := Leds.make_rgb(localRed, localGreen, localBlue)
    
PUB DimLong(localColor, localBrightness) | localRed, localGreen, localBlue
 
  if localBrightness == MAX_STRIP_BRIGHTNESS
    return localColor

  localRed := (((localColor >> 16) & $FF) * localBrightness) >> 8
  localGreen := (((localColor >> 8) & $FF) * localBrightness) >> 8
  localBlue := ((localColor & $FF) * localBrightness) >> 8
  result := Leds.make_rgb(localRed, localGreen, localBlue)
    
{pri color_shift | localLevel, color

'' Color shift all leds from red to blue and back
'' -- color written to stripOutputBuffer1 for mixing by foreground cog

  ' fade from off to deep red

  repeat localLevel from $00 to $1F                                  ' from off to deep red
    longfill(@stripOutputBuffer1, leds.make_rgb(localLevel, 0, 0), CHANNELS)
    Pause(65)  

  repeat

    ' red to blue
    
    repeat localLevel from $1F to $00            
      longfill(@stripOutputBuffer1, leds.make_rgb(localLevel, 0, $1F - localLevel), CHANNELS)
      Pause(65)

    ' blue to red 

    repeat localLevel from $00 to $1F            
      longfill(@stripOutputBuffer1, leds.make_rgb(level, 0, $1F - localLevel), CHANNELS)   
      Pause(65)
      

PRI larsen_scan | ch

'' Create Larsen scanner (Cylon, Knight Rider)
'' -- color written to cbuf2 for mixing by foreground cog

  repeat
    repeat ch from 0 to LAST_CHANNEL-1                               ' loop forward
      longfill(@cbuf2, -1, CHANNELS)                            ' remove color
      cbuf2[ch] := $1F_1F_1F                                    ' set to white
      pause(1000 / CHANNELS)                                    ' hold

    repeat ch from LAST_CHANNEL to 1                                 ' loop back
      longfill(@cbuf2, -1, CHANNELS)
      cbuf2[ch] := $1F_1F_1F
      pause(1000 / CHANNELS)
}
PUB Pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1792                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]     

PUB ClearLcd

  Lcd.Char(12)
  Pause(5)

PUB BothStr(localPtr)

  Pst.Str(localPtr)
  Lcd.Str(localPtr)
  
PUB BothDec(value, localSize, padChar)

  Pst.Dec(value)

  LcdPad(value, localSize, padChar)
   
{PUB LcdPad(value, localSize, padChar)

  result := 1
  localSize--
  repeat localSize
    result *= 10
  repeat while value < result
    Lcd.Char(padChar)
    result /= 10
  Lcd.Dec(value)
}   
PUB LcdDecClean(value, digits) | temp, localSize
' Print value to LCD and add spaces to cover up possible older digits.
' Use this when a number might be negative.

  if value < 0
    localSize := 2
  else
    localSize := 1

  Pst.Str(string(ASCII_CR, "value = "))
  Pst.Dec(value)
  Pst.Str(string(ASCII_CR, "digits = "))
  Pst.Dec(digits)
  Pst.Str(string(ASCII_CR, "localSize = "))
  Pst.Dec(localSize)
  
  temp := 10
  repeat while value => temp
    localSize++
    temp *= 10
    Pst.Str(string(ASCII_CR, "localSize = "))
    Pst.Dec(localSize)
    Pst.Str(string(ASCII_CR, "temp = "))
    Pst.Dec(temp)
  
  Lcd.Dec(value)

  digits -= localSize
  if digits > 0
    repeat localSize
      Lcd.Char(ASCII_SPACE) 
      Pst.Char("*")
      
PUB LcdPad(value, digits, padChar) | temp, padFlag, digitsLeft

  result := 1
  padFlag := 1
  digitsLeft := digits
  repeat digits 
    result *= 10
  value //= result
  result /= 10  
  repeat digits
    temp := value / result
    if temp == 0 and padFlag == 1 and --digitsLeft > 1
      Lcd.Char(padChar)
    else 'temp <> 0
      padFlag := 0
      Lcd.Char(temp + "0")
    value //= result
    result /= 10
   
PUB ZeroPad(value, digits) | temp

  result := 1
  repeat digits  ' - 1
    result *= 10
  value //= result ' truncate
  result /= 10  
  repeat digits
    temp := value / result
    Pst.Char(temp + "0")
    value //= result
    result /= 10

PUB CheckTime

  if AnyTimeChanged
    return
  if cnt - lastTimeCheck > checkTimeInterval 'halfSecond
    lastTimeCheck := cnt 'halfSecond
    Clock.readTime
    second := Clock.clockSecond
    if second == previousSecond
      return  
    year := Clock.clockYear
    month := Clock.clockMonth
    date := Clock.clockDate
    day := Clock.clockDay
    hour := Clock.clockHour
    minute := Clock.clockMinute
     
    previousSecond := second
    DisplayTime
  longmove(@editedTime, @lastReadTime, TIME_ARRAY_SIZE)

{********  
PUB DisplayTime | pmFlag
 
  ' ** add flashing digits
  ' *** make sure RTC is in military time

  
  'result := hour // 12   '*** check this
  if hour > 11 and hour <> 24
    pmFlag := 1
  else
    pmFlag := 0
 { Pst.str(string(ASCII_CR, "Date: "))   
  ZeroPad(month, 2)
  Pst.Char("/")
  ZeroPad(date, 2)
  Pst.Char("/")
  ZeroPad(year, 2) 
  Pst.str(string(ASCII_CR, "Time: "))   
  ZeroPad(result, 2)
  'ZeroPad(hour, 2)
  Pst.Char(":")
  ZeroPad(minute, 2)
  Pst.Char(":")
  ZeroPad(second, 2)
  if pmFlag == 1
    Pst.str(String(" pm"))
  else
    Pst.str(String(" am"))     }
  if dateLocation
    Lcd.Char(dateLocation)
    LcdPad(month, 2, "0")
    Lcd.Char("/") 
    LcdPad(date, 2, "0")
    Lcd.Char("/")
    LcdPad(lastReadTime[YEAR_INDEX], 2, "0")
  if timeLocation
    Lcd.Char(timeLocation)
    result := hour
    if hourMode == AM_PM_TIME ' MILITARY_TIME
      if hour > 12
        result := hour - 12    
    LcdPad(result, 2, "0")
    Lcd.Char(":") 
    LcdPad(minute, 2, "0")
    Lcd.Char(":") 
    LcdPad(second, 2, "0")
    if hourMode == AM_PM_TIME
      if pmFlag == 1
        Lcd.str(string("pm"))
      else
        Lcd.str(string("am"))
    else
      Lcd.str(string("  "))
      
********}

'**
PUB DisplayTime | pmFlag
 
  ' ** add flashing digits
  ' *** make sure RTC is in military time

  
  'result := hour // 12   '*** check this
  if hour > 11 and hour <> 24
    pmFlag := 1
  else
    pmFlag := 0
  repeat until not lockset(debugLock)
  Pst.str(string(ASCII_CR, "Date: "))   
  ZeroPad(month, 2)
  Pst.Char("/")
  ZeroPad(date, 2)
  Pst.Char("/")
  ZeroPad(year, 2) 
  Pst.str(string(ASCII_CR, "Time: "))   
  ZeroPad(result, 2)
  'ZeroPad(hour, 2)
  Pst.Char(":")
  ZeroPad(minute, 2)
  Pst.Char(":")
  ZeroPad(second, 2)
  if pmFlag == 1
    Pst.str(String(" pm"))
  else
    Pst.str(String(" am"))  
  if dateLocation
    Lcd.Char(dateLocation)
    LcdPad(month, 2, "0")
    Lcd.Char("/") 
    LcdPad(date, 2, "0")
    Lcd.Char("/")
    LcdPad(lastReadTime[YEAR_INDEX], 2, "0")
  if timeLocation
    Lcd.Char(timeLocation)
    result := hour
    if hourMode == AM_PM_TIME ' MILITARY_TIME
      if hour > 12
        result := hour - 12    
    LcdPad(result, 2, "0")
    Lcd.Char(":") 
    LcdPad(minute, 2, "0")
    Lcd.Char(":") 
    LcdPad(second, 2, "0")
    if hourMode == AM_PM_TIME
      if pmFlag == 1
        Lcd.str(string("pm"))
      else
        Lcd.str(string("am"))
    else
      Lcd.str(string("  "))
  lockclr(debugLock)  
  
'**
PUB ChangeTime

  repeat until not lockset(debugLock)
  Pst.str(string(ASCII_CR, "ChangeTime Method"))
  lockclr(debugLock)
  
  repeat result from 0 to SECOND_INDEX 'TIME_ARRAY_SIZE - 1
    if timeChangedFlag[result] == 0
      editedTime[result] := lastReadTime[result]
    else
      timeChangedFlag[result] := 0  
  'longmove(@editedTime, @lastReadTime, TIME_ARRAY_SIZE)            
  Clock.writeTime(editedTime[SECOND_INDEX], editedTime[MINUTE_INDEX], {
  } editedTime[HOUR_INDEX], editedTime[DAY_INDEX], editedTime[DATE_INDEX], {
  } editedTime[MONTH_INDEX], editedTime[YEAR_INDEX]) 
   
PUB CheckTemp

  if activeTempFlag
    if cnt - startTempTime > activeTempTime
      activeTempFlag := 0
      EraseTemp
      
PUB EraseTemp

  Lcd.Char(tempStringLocation)
  result := tempStringPtr
  repeat tempStringSize
    Lcd.Char(ASCII_SPACE)
    
PUB TempString(localPtr, localTime, localStringLocation)

  if activeTempFlag and localPtr <> tempStringPtr and localStringLocation <> tempStringLocation
    EraseTemp
  activeTempTime := localTime * MS_001
  startTempTime := cnt
  tempStringLocation := localStringLocation
  tempStringSize := strsize(localPtr)
  Lcd.Char(tempStringLocation)
  Lcd.Str(localPtr)
  
PUB SpiCheck | localIndex

  Pst.str(String(13, "Attempting to read rx address from module."))
  Nordic.SpiReadPub(Nordic#RX_ADDR_P0, @globalPayload, 5)
  Pst.str(String(13, "rx address = $"))
  repeat localIndex from 0 to 4
    Pst.hex(globalPayload[localIndex], 2)
  if CompareArrays(@globalPayload, @testAddress, 5) == -1
    Pst.str(String(13, "The present rx address is the same as the test address."))
    Pst.str(String(13, "Changing the test address from $"))
    repeat localIndex from 0 to 4
      Pst.hex(testAddress[localIndex], 2)
    Pst.str(String(", to $"))
    repeat localIndex from 0 to 4
      testAddress[localIndex] += $11
      Pst.hex(testAddress[localIndex], 2)
  Pst.str(String(13, "Attempting to write following rx address to module: $"))
  repeat localIndex from 0 to 4
    Pst.hex(testAddress[localIndex], 2)
  Nordic.SpiWritePub(Nordic#RX_ADDR_P0, @testAddress, 5)
  Pst.str(String(13, "Reading rx address from module: $"))
  Nordic.SpiReadPub(Nordic#RX_ADDR_P0, @globalPayload, 5)
  repeat localIndex from 0 to 4
    Pst.hex(testAddress[localIndex], 2)
  if CompareArrays(@globalPayload, @testAddress, 5) == -1
    Pst.str(String(13, 13, "!!!!Success!!!!")) 
    Pst.str(String(13, 13, "The rx address was successfully changed.")) 
    Pst.str(String(13, "The test rx address will not necessarily be the rx address used")) 
    Pst.str(String(13, "in communicating with other modules.", 13)) 
  else
    Pst.str(String(13, 13, "** Failure **")) 
    Pst.str(String(13, 13, "The rx address was NOT changed.")) 
    Pst.str(String(13, "The Propeller was not able to communicate with the")) 
    Pst.str(String(13, "Nordic module.", 13)) 
  
PRI CompareArrays(localPtr0, localPtr1, localSize)

  result := -1
  repeat localSize
    if byte[localPtr0++] <> byte[localPtr1++]
      result := 0
        
PUB Hex(value)

  Pst.Char("$")
  result := value >> 24 
  Pst.Hex(result, 2)    
  Pst.Char("_")
  result := value >> 16 
  Pst.Hex(result, 2)    
  Pst.Char("_")
  result := value >> 8 
  Pst.Hex(result, 2)    
  Pst.Char("_")
  Pst.Hex(value, 2)    
  
  
DAT

blinkBrightness         long Keypad#DEFAULT_BLINK_BRIGHTNESS
pulseBrightness         long Keypad#DEFAULT_PULSE_BRIGHTNESS
mode                    long Keypad#PULSE_LED
blinkNumber             long Keypad#DEFAULT_BLINK_NUMBER
pulseOn                 long Keypad#DEFAULT_PULSE_ON * MS_001
pulseOff                long Keypad#DEFAULT_PULSE_OFF * MS_001
blinkOn                 long Keypad#DEFAULT_BLINK_ON * MS_001
blinkOff                long Keypad#DEFAULT_BLINK_OFF * MS_001
blinkPause              long Keypad#DEFAULT_BLINK_PAUSE * MS_001

'parameter limits           min, max, small change, large change
pulseBLimits            long 0, Keypad#MAX_BRIGHTNESS, LITTLE_JUMP, BIG_JUMP
blinkBLimits            long 0, Keypad#MAX_BRIGHTNESS, LITTLE_JUMP, BIG_JUMP
pulseOnLimits           long Keypad#DEFAULT_PULSE_ON / 10, Keypad#DEFAULT_PULSE_ON * 10, 2, 10
pulseOffLimits          long Keypad#DEFAULT_PULSE_OFF / 10, Keypad#DEFAULT_PULSE_OFF * 10, 2, 10
blinkOnLimits           long Keypad#DEFAULT_BLINK_ON / 10, Keypad#DEFAULT_BLINK_ON * 10, 2, 10
blinkOffLimits          long Keypad#DEFAULT_BLINK_OFF / 10, Keypad#DEFAULT_BLINK_OFF * 10, 2, 10
pauseLimits             long Keypad#DEFAULT_BLINK_PAUSE / 10, Keypad#DEFAULT_BLINK_PAUSE * 10, 2, 10

radioVolume             long DEFAULT_RADIO_VOLUME
radioVLimits            long MIN_RADIO_VOLUME, MAX_RADIO_VOLUME
                        long SMALL_CHANGE_RADIO_VOLUME, BIG_CHANGE_RADIO_VOLUME
radioSteps              long 50
radioStepsLimits        long MIN_RADIO_STEPS, MAX_RADIO_STEPS
                        long SMALL_CHANGE_RADIO_STEPS, BIG_CHANGE_RADIO_STEPS

xmasDelay               long DEFAULT_XMAS_SPEED
xmasDelayLimits         long MIN_XMAS_SPEED
                        long MAX_XMAS_SPEED, 1, 10

stripSpeed              long DEFAULT_STRIP_SPEED
stripSpeedLimits        long MIN_STRIP_SPEED
                        long MAX_STRIP_SPEED, 1, 10 

stripSize               long DEFAULT_STRIP_SIZE
stripSizeLimits         long 0, 16, 1, 2        ' The max strip size is dynamically changed
                                                ' based on the pattern size.
                                                ' The stripSize indicates how many
                                                ' LEDs to insert between each element of
                                                ' a pattern.
  ' displayType enumeration
' #0, PATTERN_DISPLAY, LARSEN_DISPLAY, COLOR_DROP_DISPLAY, OFF_DISPLAY

displayType             long PATTERN_DISPLAY
displayTypeLimits       long PATTERN_DISPLAY, OFF_DISPLAY, 1, 1

stripBrightness         long DEFAULT_STRIP_BRIGHTNESS
stripBrightnessLimits   long 0, MAX_STRIP_BRIGHTNESS, 1, 20

larsenBackIndex         long OFF_STRIP
larsenBackIndexLimits   long OFF_STRIP, INDIGO_STRIP, 1, 2 

larsenForeIndex         long RED_STRIP
larsenForeIndexLimits   long RED_STRIP, INDIGO_STRIP, 1, 2 

stripColor              long RAINBOW_STRIP
stripColorLimits        long RED_STRIP, RAINBOW_STRIP, 1, 2

larsensInUse            long DEFAULT_LARSEN_SCANNERS
larsensInUseLimits      long MIN_LARSEN_SCANNERS, MAX_LARSEN_SCANNERS
                        long 1, 4
                        
larsenSpeed             long DEFAULT_STRIP_SPEED
larsenSpeedLimits       long MIN_STRIP_SPEED
                        long MAX_STRIP_SPEED, 1, 10 

larsenBrightness        long DEFAULT_STRIP_BRIGHTNESS
larsenBrightnessLimits  long 0, MAX_STRIP_BRIGHTNESS, 1, 20

larsenSize              long DEFAULT_LARSEN_SIZE
larsenSizeLimits        long MIN_LARSEN_SIZE, MAX_LARSEN_SIZE
                        long 1, 4

xmasSize                long 8
xmasSizeLimits          long 0, 80, 1, 6


xmasBrightness          long DEFAULT_XMAS_BRIGHTNESS
xmasBLimits             long 0, MAX_XMAS_BRIGHTNESS, 1, 20

yearLimits              long 2000, 2099, 1, 5
monthLimits             long 1, 12, 1, 3
dateLimits              long 0, 31, 1, 10
dayLimits               long 1, 7, 1, 3
hourLimits              long 0, 23, 1, 5
timeLimits              long 0, 59, 1, 10
hourModeLimits          long AM_PM_TIME, MILITARY_TIME, 1, 1

lastReadTime
year                    long 0
month                   long 0
date                    long 0
day                     long 0
hour                    long 0
minute                  long 0
second                  long 0
    
DAT

'background              long $00_00_00_00
'blackSetting            long $00_00_00_00      
background              byte 0[3]
blackSetting            byte 0[3]    
redSetting              byte RED_R, RED_G, RED_B
orangeSetting           byte ORANGE_R, ORANGE_G, ORANGE_B
yellowSetting           byte YELLOW_R, YELLOW_G, YELLOW_B
greenSetting            byte GREEN_R, GREEN_G, GREEN_B
blueSetting             byte BLUE_R, BLUE_G, BLUE_B
violetSetting           byte VIOLET_R, VIOLET_G, VIOLET_B
whiteSetting            byte 255, 255, 255 'WHITE_R, WHITE_G, WHITE_B
indigoSetting           byte INDIGO_R, INDIGO_G, INDIGO_B

backgroundL             long $00_00_00_00
blackSettingL           long $00_00_00_00      
redSettingL             long Leds#RED
orangeSettingL          long ORANGE_COLOR 
yellowSettingL          long YELLOW_COLOR 'Leds#YELLOW
greenSettingL           long Leds#GREEN
blueSettingL            long Leds#BLUE
violetSettingL          long Leds#VIOLET
whiteSettingL           long Leds#WHITE
indigoSettingL          long INDIGO_COLOR

DAT 'speed to delay lookup table

speedToDelay            long 2000, 1951, 1904, 1857, 1811, 1765, 1721, 1677, 1635, 1593
                        long 1551, 1511, 1471, 1432, 1394, 1357, 1320, 1284, 1249, 1214
                        long 1180, 1147, 1115, 1083, 1051, 1021, 991, 961, 933, 904
                        long 877, 850, 824, 798, 772, 748, 723, 700, 677, 654
                        long 632, 610, 589, 569, 549, 529, 510, 491, 473, 455
                        long 438, 421, 404, 388, 373, 358, 343, 328, 314, 301
                        long 287, 275, 262, 250, 238, 227, 216, 205, 194, 184
                        long 175, 165, 156, 147, 139, 131, 123, 115, 108, 101
                        long 94, 88, 82, 76, 70, 65, 59, 55, 50, 46
                        long 41, 37, 34, 30, 27, 24, 21, 18, 16, 14
extraJustIncase         long 12, 10, 8, 6, 4, 3, 2, 1, 0
DAT

                                       '1         2         3
                             '01234567890123456789012345678901
'packetTemplate          byte "what is $00 + $00?", 13, 0[14]
packetTemplate          byte $FF[4], 0
packet                  byte $FF[4], 0

nibbleTemplate          byte $0F[8], 0

testAddress   byte $AA, $BB, $CC, $DD, $EE


''------------------ Cofig settings for new nRF24L01+ modules ----------------------------------------
'' ----------------- Keep below together and in order ------------------------------------------------
config        byte $78 - %0100_0000 + %0000_0011' add irq, power up and RRX
                 ' $78 = %0111_1000
              byte 0
              byte %0000_0011
              byte $03 ' %0000_0011 ' five digit address
              byte $00 ' %0000_0000
              byte $02 ' %0000_0010
              byte $07 ' %0000_0111 ' 1Mbit, 0dBm power, zero bit don't care same as $06
              
nordicStatus  byte %01000000 ' $40   %01110000   '$70
              ' only zero allowed, Data Ready RX, TX ACK received, MAX_RT failed,
  
rxPayload     byte FOB_PAYLOAD_SIZE '_RxPayload '25 '32   ' Register $11 RX_PW_P0

rxAddress     byte $E7[5] ' Register $10 Transmit address data pipe 0, LSByte first
txAddress     byte $E7[5] ' Register $10 Transmit address data pipe 0, LSByte first
'' ----------------- Keep above together and in order ------------------------------------------------

DAT ' Nordic Fob Button Info
  ' button enumeration
'  #0, NO_BUTTON, CENTER_BUTTON, UP_BUTTON, DOWN_BUTTON, LEFT_BUTTON, RIGHT_BUTTON


fobButtonMask           long %0_0000
centerButton            long %1_0000
upButton                long %0_0001
downButton              long %0_0010
leftButton              long %0_0100
rightButton             long %0_1000


DAT

rainbowPattern          long Leds#RED, ORANGE_COLOR, YELLOW_COLOR, Leds#GREEN, Leds#BLUE
                        long Leds#VIOLET, TERMINATOR

patternSingle           long TERMINATOR[SINGLE_COLOR_PAT_SIZE + 1]              
'pattern0      long Leds#RED, Leds#RED, Leds#RED, Leds#GREEN, Leds#GREEN, Leds#GREEN, Leds#GREEN, TERMINATOR
'pattern0      long Leds#RED, Leds#RED, Leds#RED, Leds#GREEN, Leds#GREEN, Leds#GREEN, Leds#BLUE, Leds#BLUE, Leds#BLUE, TERMINATOR
'pattern0      long Leds#RED, Leds#GREEN, Leds#GREEN, Leds#BLUE, Leds#BLUE, Leds#BLUE, TERMINATOR
   
{colorRed      long 0'$3F_00_00
              'long $3F_00_00
              long $3F_0F_00
              long 0'$3F_2F_00
              'long $3F_00_00
              'long $00_3F_00
              long 0'$3F_00_00
              'long $3F_00_00
             
'colorGreen    long $00_3F_00
              
colorOrange   long $3F_0F_00
colorYellow   long 0'$3F_2F_00

colorRed      long $3F_00_00
              'long $3F_00_00
              long $3F_0F_00
              long $3F_2F_00
              'long $3F_00_00
              'long $00_3F_00
              long $3F_00_00
              'long $3F_00_00
              
'colorGreen    long $00_3F_00
              
colorOrange   long $3F_0F_00
colorYellow   long $3F_2F_00


colorBlue     long $00_00_3F
'colorIndigo   long $00_3F_3F
colorViolet   long $3F_00_3F
colorWhilte   long $3F_3F_3F
}
{
colorRed      long $3F_00_00
colorOrange   long $3F_0F_00
colorYellow   long $3F_3F_00
colorGreen    long $00_3F_00
colorBlue     long $00_00_3F
'colorIndigo   long $00_3F_3F
colorViolet   long $3F_00_3F
colorWhilte   long $3F_3F_3F

colorRed      long $1F_00_00
colorOrange   long $1F_07_00
colorYellow   long $1F_1F_00
colorGreen    long $00_1F_00
colorBlue     long $00_00_1F
'colorIndigo   long $00_1F_1F
colorViolet   long $1F_00_1F
colorWhilte   long $1F_1F_1F

colorBlack    long $00_00_00
colorRed      long $FF_00_00
colorOrange   long $FF_3F_00
colorYellow   long $FF_FF_00
colorGreen    long $00_FF_00
colorBlue     long $00_00_FF
'colorIndigo   long $00_FF_FF
colorViolet   long $FF_00_FF
colorWhilte   long $FF_FF_FF
}
colorBlack    long $00_00_00
colorRed      long $7F_00_00
colorOrange   long $7F_1F_00
colorYellow   long $7F_7F_00
colorGreen    long $00_7F_00
colorBlue     long $00_00_7F
colorIndigo   long $00_7F_7F
colorViolet   long $7F_00_7F
colorWhilte   long $7F_7F_7F

color1        long $02_00_00
              long $02_01_00
              long $02_02_00
              long $00_02_00
              long $00_00_02
              long $00_02_02
              long $02_00_02
              long $02_02_02

colorIndex    long 0[3]      
DAT

am_pm         byte "am/pm", 0
military      byte "military", 0
on_text       byte "on ", 0
off_text      byte "off", 0

DAT
{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}  